﻿(function () {
    "use strict";

    var logMessagesPerQuery = 50;

    angular
        .module("portalApp.controllers")
        .controller("LogsController", LogsController);

    LogsController.$inject = ["$rootScope", "$scope", "portalService", "signalRService", "$translate", "$timeout", "maxLogMessages", "stringFormatFilter", "$q"];

    function LogsController($rootScope, $scope, portalService, signalRService, $translate, $timeout, maxLogMessages, stringFormatFilter, $q) {
        var desiredSeverity = 7; // error + warning + info
        var vm = this;

        if ($translate.isReady()) { // Don't bother initializing the grid running unit tests
            $scope.gridOptions = {
                enableSorting: false,
                enableColumnResizing: true,
                enableColumnMenus: false,
                columnDefs: [
                    { displayName: "STE_LABEL_Time", field: "Timestamp", headerCellFilter: "translate", width: "10%", enableColumnResizing: true },
                    { displayName: "STE_LABEL_Severity", field: "Severity", headerCellFilter: "translate", width: "10%", enableColumnResizing: true },
                    { displayName: "STE_LABEL_Source", field: "Source", headerCellFilter: "translate", width: "15%", enableColumnResizing: true },
                    { displayName: "STE_LABEL_Message", field: "Message", headerCellFilter: "translate", width: "50%", enableColumnResizing: true },
                    { displayName: "STE_LABEL_Exception", field: "ExceptionText", headerCellFilter: "translate", width: "15%", enableColumnResizing: true }
                ],
                infiniteScrollDown: true,
                onRegisterApi: function (gridApi) {
                    gridApi.infiniteScroll.on.needLoadMoreData($scope, vm.loadMoreLogs);
                    $scope.gridApi = gridApi;
                },
                data: 'logs'
            };
        }

        vm.title = "LogsController";
        vm.logEntries = [];
        vm.isDownloading = false;
        vm.loadMoreLogs = loadMoreLogs;
        vm.search = search;
        vm.updateLabelsFromLocale = updateLabelsFromLocale;
        vm.timeSearchOptions = [
            { displayMember: "Last hour", value: "moment().subtract(1, 'hours')" },
            { displayMember: "Last 2 hours", value: "moment().subtract(2, 'hours')" },
            { displayMember: "Last 24 hours", value: "moment().subtract(24, 'hours')" },
            { displayMember: "Last 48 hours", value: "moment().subtract(48, 'hours')" },
            { displayMember: "Last week", value: "moment().subtract(1, 'weeks')" },
            { displayMember: "All", value: "moment().subtract(100, 'years')" }
        ];

        vm.timeSearchOption = vm.timeSearchOptions[2]; // last 24 hours
        vm.severity = {
            error: { selected: (desiredSeverity & 4) === 4, value: 4, name: "Error"},
            warning: { selected: (desiredSeverity & 2) === 2, value: 2, name: "Warning" },
            info: { selected: (desiredSeverity & 1) === 1, value: 1, name: "Information" },
            debug: { selected: (desiredSeverity & 16) === 16, value: 16, name: "Debug" },
            perf: { selected: (desiredSeverity & 8) === 8, value: 8, name: "Performance" }
        }
        vm.searchCriteria = {
            Distinct: false,
            ExceptionText: "",
            LogType: 0,
            LowerTimestamp: moment().subtract(24, 'hours'), // last 24 hours
            Message: "",
            Severity: desiredSeverity,
            Source: "",
            Top: maxLogMessages, // limit to max 5000 items
            UpperTimestamp: moment()
        };

        vm.updatedSearchOption = updatedSearchOption;
        vm.updateSourceRegistration = updateSourceRegistration;
        vm.toggle = toggle;
        vm.setupSources = setupSources;
        vm.showStickyAlert = showStickyAlert;
        vm.download = download;
        vm.timeout = undefined;
        vm.alert = { type: 'alert-warning alert-sticky fade', msg: '' };

        vm.timeFormat = "YYYY-MM-DD HH:mm:ss"; // until overwritten by user configuration

        $rootScope.isLoading = false;
        $rootScope.connectionLost = false;
        $rootScope.reconnecting = false;
        $rootScope.activated = true;

        $scope.$on("$destroy", function () {
            vm.unregisterLastActivities();
            vm.unregisterAddEntry();
            vm.unregisterTranslateChangeSuccess();
            vm.connectionErrorHandler();
            vm.connectionSuccessHandler();

            if (vm.timeout) {
                $timeout.cancel(vm.timeout);
            }
        });

        signalRService.initializeLogHub().then(function () {
            vm.setupSources();
        });

        var defer = $q.defer();

        // Call initialization functions
        activate()
            .then(function (response) {
                if (response.data.DateTimeFormat !== 1)
                    vm.timeFormat = "YYYY-MM-DD HH:mm:ss"; // international format
                else
                    vm.timeFormat = "MM/DD/YYYY h:mm:ss A"; // imperial format
        });

        function activate() {
            var request = function () {
                portalService.getUserConfig()
                    .then(function success(response) {
                        $rootScope.isLoading = false;
                        $rootScope.connectionLost = false;
                        $rootScope.reconnecting = false;
                        $rootScope.activated = true;

                        defer.resolve(response);
                    }, function error() {
                        $rootScope.connectionLost = true;
                        $rootScope.isLoading = false;

                        // wait 5 seconds then retry
                        vm.timeout = $timeout(function () {
                            $rootScope.reconnecting = true;
                            request();
                        }, 5000);
                    });
            };
            
            request();

            return defer.promise;
        }

        function setupSources() {
            // Fetch the list of log sources
            signalRService.getSources().then(function (sources) {
                sources.forEach(function (source) {
                    source.IsActive = moment().utc().isBefore(moment(source.ExpiresOn));
                });
                vm.sources = sources;
            });

            // need to use timeout here to avoid digest already in progress error in tests.
            $timeout(function () {
                // Fetch the logs corresponding to the search criterias
                search();
            });
        }

        function isSeveritySelected(messageSeverity)
        {
            var selected = false;
            angular.forEach(vm.severity, function (value, key) {
                if (value.selected && value.name === messageSeverity) {
                    selected = true;
                }
            });

            return selected;
        }       

        // Called whenever the UI list needs to display more entries
        function loadMoreLogs() {
            if ($scope.logs) {
                $scope.logs = vm.logEntries.slice(0, $scope.logs.length + logMessagesPerQuery); // fetch next few messages
                if ($scope.gridApi) {
                    $scope.gridApi.infiniteScroll.saveScrollPercentage();
                    $scope.gridApi.infiniteScroll.dataLoaded();
                }
            }
        };

        function search() {
            vm.isSearching = true; // raise the flag for the spinner
            vm.logEntries = []; // reset the list
            $scope.logs = []; // clear logs

            vm.searchCriteria.LowerTimestamp = eval(vm.timeSearchOption.value);
            var severity = 0;
            angular.forEach(vm.severity, function (value, key) {
                if (value.selected)
                    severity += value.value;
            });

            vm.searchCriteria.Severity = severity;
            vm.searchCriteria.UpperTimestamp = moment();

            signalRService.getLogEntries(vm.searchCriteria)
                .then(function (logs) {
                    // success
                    vm.isSearching = false;
                    logs.forEach(function (log) {
                        log.Timestamp = moment(log.Timestamp).format(vm.timeFormat);
                    });
                    vm.logEntries = logs;
                    $timeout(function () {
                        $scope.logs = vm.logEntries.slice(0, logMessagesPerQuery); // display few most recent logs first
                    });                    
                    
                    if(logs.length >= 5000)
                        $translate("STE_SENTENCE_TooManyResults")
							.then(function (value) {
							    vm.showStickyAlert('warning', stringFormatFilter(value, [maxLogMessages]), true);
							});
                }, function() {
                    //failure
                    vm.isSearching = false;
                    $rootScope.connectionLost = true;
                });
        }

        function updatedSearchOption(option) {
            vm.timeSearchOption = option;
        }

        function updateSourceRegistration(logSource) {
            if (logSource.IsActive)
                signalRService.enableSource(logSource.Name)
                    .then(function success() {
                        $rootScope.connectionLost = false;
                        },
                        function failed(error) {
                            $rootScope.connectionLost = true;
                        }
                    );
            else
                signalRService.disableSource(logSource.Name)
                    .then(function success() {
                        $rootScope.connectionLost = false;
                        },
                        function failed(error) {
                            $rootScope.connectionLost = true;
                        }
                    );
        }

        function toggle() {
            if (vm.isPaused) {
                signalRService.resumeLogs()
                    .then(function success() {
                        $rootScope.connectionLost = false;
                        },
                        function failed(error) {
                            $rootScope.connectionLost = true;
                        }
                    );
                vm.isPaused = false;
                search();
            } else {
                signalRService.pauseLogs()
                    .then(function success() {
                        $rootScope.connectionLost = false;
                        },
                        function failed(error) {
                            $rootScope.connectionLost = true;
                        }
                    );
                vm.isPaused = true;
            }
        }

        function showStickyAlert(type, msg, timeout, continueWith) {
            vm.alert = { type: type, msg: msg }
            vm.alert.type = 'alert-' + type + ' alert-sticky fade in';
            if (timeout) {
                if (vm.timeout) {
                    $timeout.cancel(vm.timeout);
                }

                vm.timeout = $timeout(function () {
                    vm.alert.type = 'alert-' + type + ' alert-sticky fade';

                    if (!!continueWith && typeof continueWith == "function") {
                        continueWith();
                    }

                }, 5000);
            }
        }

        // Last activities hub
        signalRService.initializeLastActivitiesHub();

        // Capture "lastActivities" message and use last unit reboot and last software reboot information for filter
        vm.unregisterLastActivities = $rootScope.$on("LastActivities", function (e, message) {
            $rootScope.$apply(function () {
                if (vm.timeSearchOptions.length < 7) {
                    $translate("STE_LABEL_SinceLastUnitReboot").then(function(translation) {
                        // Add an entry at the end of the array
                        vm.timeSearchOptions.push({ displayMember: translation, value: "moment(\"" + message.LastReboot + "\")" });
                    });
                    $translate("STE_LABEL_SinceLastSoftRestart").then(function (translation) {
                        // Add an entry at the end of the array
                        vm.timeSearchOptions.push({ displayMember: translation, value: "moment(\"" + message.LastAppRestart + "\")" });
                    });
                } else {
                    // Update last entries
                    vm.timeSearchOptions[6].value = "moment(\"" + message.LastReboot + "\")";
                    vm.timeSearchOptions[7].value = "moment(\"" + message.LastAppRestart + "\")";
                }
            });
        });

        // Capture entries generated after serach was called (auto-update list)
        vm.unregisterAddEntry = $rootScope.$on("addEntry", function (e, message) {
            if (matchFilters(vm.searchCriteria, message)) {
                $rootScope.$apply(function () {
                    message.Timestamp = moment(message.Timestamp, "YYYY-MM-DD hh:mm:ss a").format(vm.timeFormat);
                    vm.logEntries.unshift(message); // add at the beginning of the array
                    vm.logEntries = vm.logEntries.slice(0, maxLogMessages); // never go above the maximum number of messages

                    if (!$scope.logs)
                        $scope.logs = [];

                    if ($scope.logs.unshift(message) > maxLogMessages) // add at the beginning of the view
                        $scope.logs.slice(0, maxLogMessages); // truncate if necessary
                });
            }
            else {
                console.log("Log: filtering out the received message: [" + message.Severity + "] " + message.Source + " '" + message.Message + "'");
            }
        });

        // Returns true if the given message matches the filter criteria (must be retained and displayed)
        function matchFilters(criteria, message) {
            if (!isSeveritySelected(message.Severity)) // check severity first
                return false;

            if (criteria.Source && criteria.Source.length > 0) // Source filter is set
                if (criteria.Source !== message.Source)
                    return false;

            if (criteria.Message && criteria.Message.length > 0) // Message filter is set
                if (!message.Message.toLowerCase().includes(criteria.Message.toLowerCase()))
                    return false;

            // time criterion always matches for now (as the time range is not implemented yet)

            return true;
        }

        if ($translate.isReady()) {
            vm.updateLabelsFromLocale();
        }

        // Translate the time search items once the translation is availble.
        vm.unregisterTranslateChangeSuccess = $rootScope.$on('$translateChangeSuccess', function () {
            vm.updateLabelsFromLocale();
        });

        vm.connectionErrorHandler = $rootScope.$on('LoggerServiceConnectionFailure', function () {
            $rootScope.connectionLost = true;
        });

        vm.connectionSuccessHandler = $rootScope.$on('LoggerServiceConnectionSuccess', function () {
            $rootScope.isLoading = false;
            $rootScope.connectionLost = false;
            $rootScope.reconnecting = false;
            vm.setupSources();
        });

        function updateLabelsFromLocale() {
            vm.timeSearchOptions[0].displayMember = $translate.instant("STE_LABEL_LastHour");
            vm.timeSearchOptions[1].displayMember = $translate.instant("STE_LABEL_Last2Hours");
            vm.timeSearchOptions[2].displayMember = $translate.instant("STE_LABEL_Last24Hours");
            vm.timeSearchOptions[3].displayMember = $translate.instant("STE_LABEL_Last48Hours");
            vm.timeSearchOptions[4].displayMember = $translate.instant("STE_LABEL_LastWeek");
            vm.timeSearchOptions[5].displayMember = $translate.instant("STE_LABEL_AnyTime");
        }

        function download() {
            vm.isDownloading = true;
            var searchCriteria = {};

            searchCriteria.LowerTimestamp = moment().subtract(10, 'years'); // recover all logs
            var severity = 0;
            angular.forEach(vm.severity, function (value, key) {
                if (value.selected)
                    severity += value.value;
            });

            searchCriteria.Severity = severity;
            searchCriteria.UpperTimestamp = moment(); // until now

            signalRService.getXMLLogEntries(vm.searchCriteria)
                .then(function (result) {
                    vm.isDownloading = false;
                    var date = moment().format('YYYY-MM-DD_HH:mm:ss');
                    var fName = "Logs-" + date + ".xml";
                    var file = new Blob([result], { type: 'application/xml' });
                    var fileUrl = (window.URL || window.webkitURL).createObjectURL(file);

                    if (navigator.msSaveBlob) {
                        navigator.msSaveBlob(file, fName);
                    }
                    else {
                        // try to get the invisble element
                        var downloadLink = document.getElementById("logsDownload");
                        if (!downloadLink) {
                            // we create an invisible a element to do the download in the browser
                            downloadLink = document.createElement("a");
                            downloadLink.setAttribute("id", "logsDownload");
                            document.body.appendChild(downloadLink);
                            downloadLink.style = "display: none";
                        }

                        downloadLink.href = fileUrl;
                        downloadLink.download = fName;
                        downloadLink.click();
                    }
                }, function() {
                    //error
                    vm.isDownloading = false;
            });
        }
    }
})();
