﻿(function () {
    "use strict";

    angular
        .module("portalApp.controllers")
        .controller("TimeController", TimeController);

    TimeController.$inject = ["portalService", "$rootScope", "$scope", "$timeout", "$filter", "$translate", "stringFormatFilter", "$uibModal", "checkChangeStateFactory", "$interval", "$q"];

    function TimeController(portalService, $rootScope, $scope, $timeout, $filter, $translate, stringFormatFilter, $uibModal, checkChangeStateFactory, $interval, $q) {
        var vm = this;
        vm.class = "TimeController";
        vm.save = save;
        vm.cancel = cancel;
        vm.show = false;
        vm.alert = {};
        vm.openTimeSyncModal = openTimeSyncModal;
        vm.isSaving = false;
        vm.tickInterval = 1000; //ms
        vm.tick = tick;
        vm.time = "Acquiring time..."; // placeholder
        vm.TestNtp = TestNtp;
        vm.DriftTolerance = 5000;
        vm.selectedExtension = { Name: "None", CanSyncTime: true }; // placeholder
        vm.timerSync = undefined;
        vm.timeout = undefined;
        var defer = $q.defer();

        $scope.$on("$destroy", function () {
            $timeout.cancel(vm.timeout);
            defer.reject();
            $timeout.cancel(vm.timer);
            if (vm.timerSync) {
                $interval.cancel(vm.timerSync);
            }
        });

        $rootScope.isLoading = true;
        $rootScope.connectionLost = false;
        $rootScope.reconnecting = false;
        $rootScope.activated = false;
        
        activate()
            .then(function (response) {
                vm.extensions = response.data;

                for (var i = 0; i < vm.extensions.length; i++) {
                    if (vm.extensions[i].Guid === vm.userConfig.CurrentExtension) {
                        vm.selectedExtension = vm.extensions[i];
                        break;
                    }
                }

                evaluateDrift();

                // Start the timer
                $timeout(vm.tick, vm.tickInterval);
            });

        function activate() {
            var request = function () {
                if ($rootScope.reconnecting)
                    $rootScope.activated = true;

                portalService.getUserConfig()
                    .then(function success(response) {
                        vm.userConfig = response.data;
                        // create a copy for canceling
                        vm.backup = angular.copy(vm.userConfig);

                        portalService.getExtensions()
                            .then(function success(response) {
                                $rootScope.isLoading = false;
                                $rootScope.connectionLost = false;
                                $rootScope.reconnecting = false;
                                $rootScope.activated = true;
                                // we resolve only when the second call is completed
                                defer.resolve(response);
                            }, function error() {
                                $rootScope.connectionLost = true;
                                $rootScope.isLoading = false;
                                // wait 5 seconds then retry
                                $timeout(function () {
                                    $rootScope.reconnecting = true;
                                    request();
                                }, 5000);
                            });

                    }, function error() {
                        $rootScope.connectionLost = true;
                        $rootScope.isLoading = false;
                        // wait 5 seconds then retry
                        vm.timeout = $timeout(function () {
                            $rootScope.reconnecting = true;
                            request();
                        }, 5000);
                    });
            }

            request();

            return defer.promise;
        }

        checkChangeStateFactory.checkFormOnStateChange($scope);

        $scope.closeModal = function () {
            $scope.modalInstance.dismiss("cancel");
            $interval.cancel(vm.timerSync);
        };

        $translate("STE_LABEL_UnknownDrift").then(function (value) {
            vm.message = value;
        });           

        // Display the current time in proper format
        function tick() {
            portalService.getTime()
                .then(function (result) {
                    var m = moment(result.data);
                    if (vm.userConfig.DateTimeFormat == 1) // imperial format
                        vm.time = m.format('MM/DD/YYYY h:mm:ss A');
                    else // international format
                        vm.time = m.format('YYYY-MM-DD HH:mm:ss');
                });
            vm.timer = $timeout(vm.tick, vm.tickInterval); // reset the timer
        }        

        function setTimeForSync() {
            //var utc = new Date(moment.utc().format());
            vm.newDate = moment.utc().toDate();
            vm.newDate.setMilliseconds(0);
        }

        $scope.timeChanged = function () {
            $interval.cancel(vm.timerSync);
        }

        function evaluateDrift() {
            portalService.getTime()
                .then(function (result) {
                    var now = moment.utc();
                    var sharpTime = moment.utc(result.data);
                    var drift = now - sharpTime;

                    if (drift > vm.DriftTolerance) {
                        $translate("STE_LABEL_ClockIsLagging").then(function (value) {
                            vm.message = stringFormatFilter(value, [drift / 1000]);
                            vm.messageClass = "text-warning";
                        });
                    }
                    else if (-drift > vm.DriftTolerance){
                        $translate("STE_LABEL_ClockIsLeading").then(function (value) {
                            vm.message = stringFormatFilter(value, [drift / 1000]);
                            vm.messageClass = "text-warning";
                        });
                    }
                    else {
                        $translate("STE_LABEL_ClockIsSynchronized").then(function (value) {
                            vm.message = value;
                            vm.messageClass = "text-success";
                        });                        
                    }                        
                });
        }

        function save() {
            vm.isSaving = true;
            portalService.setUserConfig(vm.userConfig)
                .then(function() {
                    showAlert('success', $translate.instant("STE_SENTENCE_ConfigurationSavedSuccessfully"));
                    vm.backup = angular.copy(vm.userConfig); // refresh the backup snapshot
                    vm.form.$setPristine();
                }, function() {
                    //error
                    showAlert('danger', $translate.instant("STE_SENTENCE_CouldNotSaveConfiguration"));
                }).finally(function() {
                    vm.isSaving = false;
                });
        }

        function TestNtp() {
            portalService.testNtp(vm.userConfig.TimeSyncSettings.TimeServer)
                .then(function(result) {
                    if (result.data === true) {
                        showAlert('success', $translate.instant("STE_SENTENCE_ConnectionTestSucceeded"));
                    } else {
                        showAlert('danger', $translate.instant("STE_SENTENCE_ConnectionTestFailed"));
                    }
                });
        }

        function cancel() {
            vm.userConfig = angular.copy(vm.backup); // revert to backup values
            vm.form.$setPristine();
        }

        // type can be: success, info, warning or danger
        // msg is a string
        function showAlert(type, msg) {
            vm.show = true;
            vm.alert = { type: type, msg: msg }
            $timeout(function () {
                vm.show = false;
            }, 5000);
        }

        function openTimeSyncModal() {
            vm.timerSync = $interval(setTimeForSync, vm.tickInterval);
            $scope.modalInstance = $uibModal.open({
                templateUrl: "timesync.html",
                scope: $scope,
                size: "md"
            });
        }

        $scope.setTime = function () {
            var momentDate = moment.utc(vm.newDate);
            portalService.setTime(momentDate.format('YYYY-MM-DD HH:mm:ss'))
                .then(function () {
                    $scope.modalInstance.close();
                    evaluateDrift();
                }, function() {
                    $scope.modalInstance.dismiss("cancel");
                    $interval.cancel(vm.timerSync);
                });                
        }        
    }
})();
