﻿(function () {
    "use strict";

    angular
        .module("portalApp.controllers")
        .controller("VehicleDetectionController", VehicleDetectionController);

    VehicleDetectionController.$inject = ["portalService", "signalRService", "$scope", "$rootScope", "$timeout", "checkChangeStateFactory", "stringFormatFilter", "$translate", "$q", "$state", "$interval", "$window"];

    function VehicleDetectionController(portalService, signalRService, $scope, $rootScope, $timeout, checkChangeStateFactory, stringFormatFilter, $translate, $q, $state, $interval, $window) {
        var vm = this;
        vm.title = "VehicleDetectionController";
        vm.canBeActivated = false;
        vm.show = false;
        vm.alert = undefined;
        vm.save = save;
        vm.cancel = cancel;
        vm.showStickyAlert = showStickyAlert;

        vm.readyToCalibrate = false; // false until fov is set (zone is optional)
        vm.calibrating = false;
        vm.calibrationStateProgressClass = 'progress-bar-success';
        vm.calibrationStateRatio = 0;
        vm.requiredVehicles = 5;
        vm.remainingVehicles = vm.requiredVehicles;
        vm.remainingVehiclesText = "";
        vm.calibrationResult = ""; // date of end of calibration
        vm.zoneDefined = false;
        vm.restartCalibration = restartCalibration; // "Restart calibration" button clicked
        vm.virtualLoopEnabled = false;
        vm.editDetectionZone = false;
        vm.configureFovMatch = configureFovMatch; // "Configure" button clicked

        vm.updateUiFromCalibrationStatus = updateUiFromCalibrationStatus;

        vm.updateRemainingVehicles = updateRemainingVehicles;
        vm.assessReadyToCalibrate = assessReadyToCalibrate;
        vm.translate = translate;

        // Load zone usages asynchronously as $translate may not be ready for use (F5 - browser refresh sometimes goes too fast)
        vm.zoneUsages = [];
        vm.registeredTranslate = $rootScope.$on('$translateChangeSuccess', function () {
            vm.translate();
        });

        if ($translate.isReady())
            vm.translate();

        vm.getZoneName = getZoneName;
        vm.getCurrentZoneUsageName = getCurrentZoneUsageName;

        vm.userConfig = {}; // Data binding for the video feed directive
        vm.backup = {};
        vm.showZone = true; // configure the directive
        vm.showFov = true;

        $rootScope.isLoading = true;
        $rootScope.connectionLost = false;
        $rootScope.reconnecting = false;
        $rootScope.activated = false;

        var defer = $q.defer();

        // Clean up to perform when the view is closed
        $scope.$on("$destroy", function () {
            $timeout.cancel(vm.timeout);
            defer.reject();

            if (vm.statusHub) {
                vm.statusHub(); // unregister status hub
                vm.statusHub = null;
            }

            // Stop the video info feed.
            signalRService.stopFeed();
        });

        // Called when the calibration engine reports a change in calibration status
        vm.statusHub = $rootScope.$on("Status", function (e, message) {
            $rootScope.$apply(function () {
                if (message.IsCompleted)
                    console.log("VehicleDetection COMPLETED: " + message.DateTimeUtc);
                else
                    if (message.WaitingForConfiguration)
                        console.log("VehicleDetection WAITING FOR CONFIGURATION");
                    else
                        console.log("VehicleDetection PENDING. Waiting for: " + message.NbVehicleMissingForCompletingCalibration + " more vehicles");

                vm.updateUiFromCalibrationStatus(message);
            });
        });

        activate().then(function () {
            $rootScope.isLoading = false;
            $rootScope.connectionLost = false;
            $rootScope.reconnecting = false;
            $rootScope.activated = true;

            // vm.userConfig should be defined here
            // start a timer to execute function when all DOM elements are present (after reevaluating HTML view "ng-if")
            $timeout(function () { postLoading(); }, 0);
        });

        checkChangeStateFactory.checkFormOnStateChange($scope);

        function activate() {
            var request = function () {
                var promises = [
                    portalService.getProperties(),
                    portalService.getUserConfig()
                ];

                return $q.all(promises).then(
                    function success(responses) {
                        if (responses[0].data.IsIts) {
                            $rootScope.isLoading = false;
                            $rootScope.connectionLost = false;
                            $rootScope.reconnecting = false;
                            $rootScope.activated = true;

                            defer.reject();
                            return; // feature cannot be activated in ITS mode
                        }

                        vm.canBeActivated = true;

                        // set user.config default values
                        if (responses[1].data.VehicleDetectionConfiguration.ValidationRegion === null)
                            responses[1].data.VehicleDetectionConfiguration
                                .ValidationRegion = { ValidationRegionType: 2, VertexList: [] };

                        vm.userConfig = responses[1].data;
                        vm.virtualLoopEnabled = vm.userConfig.ReadModeConfiguration.ReadingMode === 3;

                        // create a copy for canceling
                        vm.backup = angular.copy(vm.userConfig);

                        // Setup channel to receive notifications about vehicle detection calibration progression
                        //!!!TODO handle error conditions
                        signalRService.initializeVehicleDetectionCalibrationHub();

                        defer.resolve();
                    },
                    function error() {
                        $rootScope.connectionLost = true;
                        $rootScope.isLoading = false;
                        // wait 5 seconds then retry
                        vm.timeout = $timeout(function () {
                            $rootScope.reconnecting = true;
                            request();
                        }, 5000);
                    }
                );
            }

            request();

            return defer.promise;
        }

        function translate() {
            vm.zoneUsages = [
                // ids must match LPR-defined enum
                { id: 0, name: $translate.instant("STE_LABEL_DetectionZoneUsage_Entering") },
                { id: 1, name: $translate.instant("STE_LABEL_DetectionZone_Exiting") },
                { id: 2, name: $translate.instant("STE_LABEL_DetectionZone_EnteringOrExiting") }
            ];
            vm.registeredTranslate(); // unregister
            vm.registeredTranslate = null;
        }

        // Perform once all operations that depend on the DOM elements to be completely initialized (including evaluating Angular ng-if)
        function postLoading() {
            // By default, stream all information
            var request = new signalRService.FeedRequestOptions(1, false, true);
            signalRService.initializeFeedHub(request);

            var vehConfig = vm.userConfig.VehicleDetectionConfiguration;

            // Only after UserConfig has been loaded...
            vm.CurrentZoneUsageName = vm.getCurrentZoneUsageName(); // store the value in a property

            vm.zoneDefined = (vehConfig.ValidationRegion !== null) ? vehConfig.ValidationRegion.VertexList.length > 2 : false;

            vm.assessReadyToCalibrate(); //!!! would normally wait for a message from the vehicle detection hub to set that value            
        }

        function updateUiFromCalibrationStatus(status) {
            vm.readyToCalibrate = !status.WaitingForConfiguration; // if we receive this message, it is because we could start the calibration
            vm.calibrating = !status.IsCompleted && !status.WaitingForConfiguration;
            vm.remainingVehicles = status.NbVehicleMissingForCompletingCalibration;

            var m = moment(status.DateTimeUtc);
            if (vm.userConfig.DateTimeFormat !== 1)
                vm.calibrationResult = m.format('YYYY-MM-DD HH:mm:ss'); // international format
            else
                vm.calibrationResult = m.format('MM/DD/YYYY h:mm:ss A'); // imperial format

            console.log("VehicleDetection trace: Configuration complete: " + vm.readyToCalibrate + ", Calibrating: " + vm.calibrating + ", Missing vehicles: " + vm.remainingVehicles + ", Results: " + vm.calibrationResult);

            var completedVehicles = vm.requiredVehicles - vm.remainingVehicles;
            vm.calibrationStateRatio = (completedVehicles / vm.requiredVehicles) * 100;

            vm.updateRemainingVehicles(vm.remainingVehicles);
        }

        function updateRemainingVehicles(nbVehicles) {
            $translate("STE_LABEL_WaitingForVehicles").then(function (value) {
                vm.remainingVehiclesText = stringFormatFilter(value, [nbVehicles]);
            });
        }

        // Called when the user clicks on the summary "Configure" button
        function configureFovMatch() {
            $state.go("main.config.lpr.vehicleDetectionEdit");
        }

        function restartCalibration() {

            if (!vm.readyToCalibrate)
                return; // preventive code

            portalService.restartVehicleDetectionCalibration()
                .then(function () {
                    // success
                    //vm.calibrating = true;
                    // we expect another message from SignalR that will update displayed values
                }, function () {
                    // failure
                    vm.showStickyAlert('danger', $translate.instant("STE_SENTENCE_CouldNotRestartCalibration"));
                });
        }

        function assessReadyToCalibrate() {
            var fovRect = vm.userConfig.VehicleDetectionConfiguration.LprFovInContextImage;
            var featureActivated = vm.backup.ReadModeConfiguration.ReadingMode === 3;
            vm.readyToCalibrate = featureActivated && (fovRect.Height > 0 && fovRect.Width > 0);
        }

        function showStickyAlert(type, msg) {
            vm.alert = {
                type: type, msg: msg
            }
            vm.show = true;
            $timeout(function () {
                vm.show = false;
            }, 5000);
        }

        function save() {
            vm.isSaving = true;

            if (vm.virtualLoopEnabled)
                vm.userConfig.ReadModeConfiguration.ReadingMode = 3; // Continuous with Vehicule detection
            else
                vm.userConfig.ReadModeConfiguration.ReadingMode = 0; // Continuous

            portalService.setUserConfig(vm.userConfig)
                .then(function () {
                    vm.showStickyAlert('success', $translate.instant("STE_SENTENCE_ConfigurationSavedSuccessfully"));
                    vm.backup = angular.copy(vm.userConfig); // refresh the backup snapshot
                    vm.assessReadyToCalibrate();
                    vm.form.$setPristine();
                }, function () {
                    //error
                    vm.showStickyAlert('danger', $translate.instant("STE_SENTENCE_CouldNotSaveConfiguration"));
                }).finally(function () {
                    vm.isSaving = false;
                });
        }

        function cancel() {
            vm.userConfig = angular.copy(vm.backup); // revert to backup values
            vm.virtualLoopEnabled = vm.userConfig.ReadModeConfiguration.ReadingMode === 3;
            vm.form.$setPristine();
        }

        function getCurrentZoneUsageName() {
            if (vm.userConfig === null || vm.userConfig.VehicleDetectionConfiguration === null) {
                console.log("UserConfig or VehicleDetectionConfiguration is null.");
                return "";
            }
            if (!vm.userConfig.VehicleDetectionConfiguration.ValidationRegion) {
                console.log("ValidationRegion is null.");
                return "";
            }

            return vm.getZoneName({
                id: vm.userConfig.VehicleDetectionConfiguration.ValidationRegion.ValidationRegionType
            });
        }

        function getZoneName(zoneUsage) {
            var id = zoneUsage.id;
            for (var i = 0; i < vm.zoneUsages.length; i++) {
                if (vm.zoneUsages[i].id === id)
                    return vm.zoneUsages[i].name;
            }

            return "";
        }

    }

})();