﻿(function () {
    "use strict";

    var lprCameraTag = "Lpr Camera";
    var contextCameraTag = "Context Camera";
    var imageSize = { width: 1280, height: 960 };

    angular
        .module("portalApp.controllers")
        .controller("VehicleDetectionEditController", VehicleDetectionEditController);

    VehicleDetectionEditController.$inject = ["portalService", "signalRService", "$scope", "$rootScope", "$timeout", "checkChangeStateFactory", "stringFormatFilter", "$translate", "$q", "$state"];

    function VehicleDetectionEditController(portalService, signalRService, $scope, $rootScope, $timeout, checkChangeStateFactory, stringFormatFilter, $translate, $q, $state) {
        var vm = this;
        vm.title = "VehicleDetectionEditController";
        vm.show = false;
        vm.alert = {};
        vm.showStickyAlert = showStickyAlert;
        vm.translate = translate;
        vm.assessFormValid = assessFormValid;

        vm.editFovMatch = true;
        vm.editDetectionZone = false;
        vm.clearZone = clearZone;
        vm.selectZoneAll = selectZoneAll;
        vm.gotoDetectionZone = gotoDetectionZone; // "Next" button clicked
        vm.gotoFovMatch = gotoFovMatch; // "Back" button clicked

        vm.isFovValid = true; // set to true initially to prevent error message from showing, but keep isZoneValid false so user can't click Next.
        vm.isZoneValid = false;

        // Load zone usages asynchronously as $translate may not be ready for use (F5 - browser refresh sometimes goes too fast)
        vm.zoneUsages = [];
        vm.registeredTranslate = $rootScope.$on('$translateChangeSuccess', function () {
            vm.translate();
        });

        if ($translate.isReady())
            vm.translate();

        vm.getZoneName = getZoneName;
        vm.getCurrentZoneUsageName = getCurrentZoneUsageName;
        vm.selectZoneUsage = selectZoneUsage;
        vm.endZoomAndFocus = endZoomAndFocus;

        vm.userConfig = {}; // Data binding for the video feed directive
        vm.backup = {};
        vm.showMarkers = true;
        vm.editMarkers = true;
        vm.showFov = true;
        vm.showZone = true;
        vm.editZone = true;

        vm.contextExposure = 50;
        vm.lprExposure = 50;
        vm.updateLprExposureSettings = updateLprExposureSettings;
        vm.updateContextExposureSettings = updateContextExposureSettings;

        vm.isSaving = false;
        vm.cancel = cancel; // "Cancel" button clicked
        vm.done = done; // "Done" button clicked

        $rootScope.isLoading = true;
        $rootScope.connectionLost = false;
        $rootScope.reconnecting = false;
        $rootScope.activated = false;

        var defer = $q.defer();

        // Watch the userConfig for changes. When a change occurs, we validate the data (zone, fov, ..)
        vm.unwatchUserConfig = $scope.$watch("vm.userConfig", function () {
            assessFormValid();
        }, true);

        // Clean up to perform when the view is closed
        $scope.$on("$destroy", function () {
            $timeout.cancel(vm.timeout);
            defer.reject();

            vm.unwatchUserConfig();
            vm.unwatchUserConfig = null;

            if (vm.registeredTranslate) {
                vm.registeredTranslate();
                vm.registeredTranslate = null;
            }
        });

        activate().then(function () {
            $rootScope.isLoading = false;
            $rootScope.connectionLost = false;
            $rootScope.reconnecting = false;
            $rootScope.activated = true;

            // start a timer to execute function when all DOM elements are present (after reevaluating HTML view "ng-if")
            $timeout(function () { postLoading(); }, 0);
        });

        // Prevent (ask for confirmation) leaving a dirty page without saving changes
        checkChangeStateFactory.checkFormOnStateChange($scope);

        function activate() {
            var request = function () {
                if ($rootScope.reconnecting)
                    $rootScope.activated = true;

                var promises = [
                    portalService.getUserConfig()
                ];

                return $q.all(promises).then(
                    function success(responses) {
                        // setting default values
                        if (responses[0].data.VehicleDetectionConfiguration.ValidationRegion === null)
                            responses[0].data.VehicleDetectionConfiguration.ValidationRegion = {
                                ValidationRegionType: 2,
                                VertexList: []
                            };

                        vm.userConfig = responses[0].data;

                        // create a copy for canceling
                        vm.backup = angular.copy(vm.userConfig);

                        defer.resolve();
                    },
                    function error() {
                        $rootScope.connectionLost = true;
                        $rootScope.isLoading = false;
                        // wait 5 seconds then retry
                        vm.timeout = $timeout(function () {
                            $rootScope.reconnecting = true;
                            request();
                        }, 5000);
                    }
                );
            }

            request();

            return defer.promise;
        }

        function translate() {
            vm.zoneUsages = [
                // ids must match LPR-defined enum
                { id: 0, name: $translate.instant("STE_LABEL_DetectionZoneUsage_Entering") },
                { id: 1, name: $translate.instant("STE_LABEL_DetectionZone_Exiting") },
                { id: 2, name: $translate.instant("STE_LABEL_DetectionZone_EnteringOrExiting") }
            ];
            vm.registeredTranslate(); // unregister
            vm.registeredTranslate = null;
        }

        // Perform once all operations that depend on the DOM elements to be completely initialized (including evaluating Angular ng-if)
        function postLoading() {
            // By default, stream all information
            var request = new signalRService.FeedRequestOptions(3, true, true);
            signalRService.initializeFeedHub(request);

            // prevent asking twice for confirmation to quit the page
            portalService.disableLogout();

            // Stop auto-exposure, allow temporarily override of exposure settings (keep a backup) and open iris to full.
            // Should be done in the back-end so that it can restore in case of problem
            portalService.beginZoomAndFocus(lprCameraTag)
                .then(function success(value) {
                    if (value.status === 200) // OK
                        vm.lprExposure = value.data.ExposureLevel * 100;
                },
                function error() {
                    showStickyAlert('warning', $translate.instant("STE_SENTENCE_STE_SENTENCE_CouldNotSuspendAutoIllumination"));
                });
            portalService.beginZoomAndFocus(contextCameraTag)
                .then(function success(value) {
                    if (value.status === 200) // OK
                        vm.contextExposure = value.data.ExposureLevel * 100;
                },
                function error() {
                    showStickyAlert('warning', $translate.instant("STE_SENTENCE_STE_SENTENCE_CouldNotSuspendAutoIllumination"));
                });
        }

        // This function wraps two calls into one so that they are always called
        function endZoomAndFocus() {
            var defer = $q.defer();
            var promises = [portalService.endZoomAndFocus(lprCameraTag), portalService.endZoomAndFocus(contextCameraTag)];
            $q.all(promises).then(function () {
                defer.resolve();
            }, function (error) {
                console.log(error);
                defer.reject();
            });
            return defer.promise;
        }

        function updateContextExposureSettings() {
            updateExposureIndex(contextCameraTag, vm.contextExposure);
        }

        function updateLprExposureSettings() {
            updateExposureIndex(lprCameraTag, vm.lprExposure);
        }

        function updateExposureIndex(cameraTag, index) {
            portalService.setFocusExposureLevel(cameraTag, index / 100, false)
                .then(function (value) {
                    // success
                }, function (value) {
                    // Failure
                    if (value && value.status === 403) {
                        // most probably user stayed too long in the page and auto-iris was restored
                        showStickyAlert('warning', $translate.instant("STE_SENTENCE_AutoIlluminationBackOn"));
                    }
                });
        }

        // Delete all points defining the zone and refresh UI
        function clearZone() {
            if (vm.userConfig.VehicleDetectionConfiguration.ValidationRegion) {
                vm.userConfig.VehicleDetectionConfiguration.ValidationRegion.VertexList = [];
                vm.form.$setDirty();
                assessFormValid();
            }
        }

        // Select whole image as the zone and refresh UI
        function selectZoneAll() {
            if (vm.userConfig.VehicleDetectionConfiguration.ValidationRegion) {
                vm.userConfig.VehicleDetectionConfiguration.ValidationRegion.VertexList = [{ "X": 0, "Y": 0 }, { "X": imageSize.width - 1, "Y": 0 }, { "X": imageSize.width - 1, "Y": imageSize.height - 1 }, { "X": 0, "Y": imageSize.height - 1 }];
                vm.form.$setDirty();
                assessFormValid();
            }

            // Set the detection mode to "enters or exits" upon clicking on "Select all" since this is the only supported mode when selecting all of the picture as detection zone.
            vm.selectZoneUsage(vm.zoneUsages[2], vm.form);
            vm.showStickyAlert('info', $translate.instant("STE_SENTENCE_AllZoneSelected").format($translate.instant("STE_LABEL_DetectionZone_EnteringOrExiting")));
        }

        function getCurrentZoneUsageName() {
            if (vm.userConfig == null || vm.userConfig.VehicleDetectionConfiguration == null)
                return "";
            if (!vm.userConfig.VehicleDetectionConfiguration.ValidationRegion)
                return "";

            return getZoneName({
                id: vm.userConfig.VehicleDetectionConfiguration.ValidationRegion.ValidationRegionType
            });
        }

        function getZoneName(zoneUsage) {
            var id = zoneUsage.id;
            for (var i = 0; i < vm.zoneUsages.length; i++) {
                if (vm.zoneUsages[i].id === id)
                    return vm.zoneUsages[i].name;
            }

            return "";
        }

        function selectZoneUsage(zoneUsage, form) {
            if (vm.userConfig.VehicleDetectionConfiguration.ValidationRegion)
                vm.userConfig.VehicleDetectionConfiguration.ValidationRegion.ValidationRegionType = zoneUsage.id;
        }

        // Called when the user clicks on the Match Fov "Next" button
        function gotoDetectionZone() {
            // Will flip elements in the view to simulate a tab change
            vm.editFovMatch = false;
            vm.editDetectionZone = true;
        }

        // Called when the user clicks on the Zone Configuration "Back" button
        function gotoFovMatch() {
            // Will flip elements in the view to simulate a tab change
            vm.editFovMatch = true;
            vm.editDetectionZone = false;
        }

        // Called when the user clicks on the Match Fov "Cancel" button, or after "Done" (save)
        function gotoSummary() {
            // auto-exposure will be restored on state transition through registeredStateChange notification

            //Stop video
            signalRService.stopFeed();
            $state.go("main.config.lpr.vehicleDetection");
        }

        function assessFormValid() {
            vm.isZoneValid = vm.userConfig && vm.userConfig.VehicleDetectionConfiguration && vm.userConfig.VehicleDetectionConfiguration.ValidationRegion.VertexList.length > 2;

            if (vm.form !== undefined) {
                vm.form.$valid = vm.isFovValid && vm.isZoneValid;
                vm.form.$invalid = !vm.form.$valid;
            }
        }

        // Save and return to summary page
        function done() {
            assessFormValid(); // Validate the form before saving, just in case

            // Return/Cancel if zone has less than 3 points defined
            if (!vm.isZoneValid) {
                vm.isSaving = false;
                return;
            }

            vm.isSaving = true;

            // let's make sure the LED stop flashing before saving the changes
            vm.endZoomAndFocus()
                .then(function () {
                    portalService.setVehDetectionConfig(vm.userConfig.VehicleDetectionConfiguration)
                        .then(function () {
                            vm.showStickyAlert('success', $translate.instant("STE_SENTENCE_ConfigurationSavedSuccessfully"));
                            vm.backup = angular.copy(vm.userConfig); // refresh the backup snapshot

                            vm.form.$setPristine();
                            gotoSummary();
                        }, function (e) {
                            //error BadRequest
                            if (e.status === 400) {// data contains enum value
                                var message = $translate.instant("STE_SENTENCE_InvalidZone");
                                switch (e.data.Message) {
                                    case "HasVertexOutsideOfContextFov":
                                        message = message + " " + $translate.instant("STE_SENTENCE_HasVertexOutsideOfContextFov");
                                        break;
                                    case "TooFarFromImageBorder":
                                        message = message + " " + $translate.instant("STE_SENTENCE_TooFarFromImageBorder");
                                        break;
                                    case "TooSmall":
                                        message = message + " " + $translate.instant("STE_SENTENCE_TooSmall");
                                        break;
                                    case "SelfIntersecting":
                                        message = message + " " + $translate.instant("STE_SENTENCE_SelfIntersecting");
                                        break;
                                    case "TooFewVertexes":
                                        message = message + " " + $translate.instant("STE_SENTENCE_TooFewVertexes");
                                        break;

                                }
                                vm.showStickyAlert('danger', message);
                            } else
                                vm.showStickyAlert('danger', $translate.instant("STE_SENTENCE_CouldNotSaveConfiguration"));
                        }).finally(function () {
                            vm.isSaving = false;
                        });
                });
        }

        function cancel() {
            vm.userConfig = angular.copy(vm.backup); // revert to backup values
            vm.form.$setPristine();
            vm.endZoomAndFocus()
                .then(function() {
                    gotoSummary(); // this is sufficient since the controller is rebuilt when changing state...
                });
        }

        function showStickyAlert(type, msg) {
            vm.alert = {
                type: type, msg: msg
            }
            vm.show = true;
            $timeout(function () {
                vm.show = false;
            }, 5000);
        }
    }
})();