﻿(function () {
    "use strict";

    angular
        .module("portalApp.controllers")
        .controller("ZoomAndFocusController", ZoomAndFocusController);

    ZoomAndFocusController.$inject = ["portalService", "$scope", "$rootScope", "$window", "$timeout", "signalRService", "$log", "checkChangeStateFactory", "$translate", "$q", "$state"];

    function ZoomAndFocusController(portalService, $scope, $rootScope, $window, $timeout, signalRService, $log, checkChangeStateFactory, $translate, $q, $state) {
        var vm = this;
        vm.title = "ZoomAndFocusController";
        vm.isAlertShown = false;
        vm.alert = {};
        vm.cameras = [];
        vm.isRulerVisible = true;
        vm.rulerHeight = 35;
        vm.editZoom = true;
        vm.userConfig = {}; // Data binding for the video feed directive

        vm.selectedCamera = undefined;
        vm.selectCamera = selectCamera;

        vm.currentExposure = undefined; // [0; 100] value will be determine when a camera is selected
        vm.canUseFlash = false;
        vm.flashEnabled = false;
        vm.resetExposure = resetExposure;
        vm.lowerExposure = lowerExposure;
        vm.raiseExposure = raiseExposure;
        vm.updateFlashEnabled = updateFlashEnabled;
        vm.isIrisFullyOpened = true; // to prevent display of the warning initially
        vm.setConfiguration = setConfiguration;
        vm.resetGraph = resetFocusGraph;
        vm.timeout = undefined;
        vm.terminate = terminate;
        vm.isTerminating = false;
        vm.terminateCompleted = terminateCompleted;
        vm.videoCanvasCursor = "default";

        $rootScope.isLoading = true;
        $rootScope.connectionLost = false;
        $rootScope.reconnecting = false;
        $rootScope.activated = false;
        var defer = $q.defer();

        activate().then(function () {
            $rootScope.isLoading = false;
            $rootScope.connectionLost = false;
            $rootScope.reconnecting = false;
            $rootScope.activated = true;

            // Listen to a zoom click event
            var zoomClicked = $rootScope.$on('onZoomClicked', resetFocusGraph);
            // Prevent memory leak by unregistering the event upon destroy
            $scope.$on('$destroy', zoomClicked);
        });

        function activate() {
            var request = function () {
                var promises = [
                    portalService.getProperties(),
                    portalService.getUserConfig()
                ];

                if ($rootScope.reconnecting)
                    $rootScope.activated = true;

                return $q.all(promises).then(
                    function success(responses) {
                        var properties = responses[0].data;
                        vm.setConfiguration(properties.IsIts);

                        vm.userConfig = responses[1].data;

                        defer.resolve();
                    },
                    function error() {
                        $rootScope.connectionLost = true;
                        $rootScope.isLoading = false;
                        // wait 5 seconds then retry
                        vm.timeout = $timeout(function () {
                            $rootScope.reconnecting = true;
                            request();
                        }, 5000);
                    });
            }

            request();

            return defer.promise;
        }

        var countDownBeforeFocusGraph = 0; // prevent update of graphic while the command modifying the focus indicator calculation has not yet been applied.

        function setConfiguration(isIts) {
            // create empty camera object
            var emptyCamera = new CameraObject("", 0, "", false);
            vm.cameras.push(emptyCamera);

            // create lpr camera object
            var lprCamera = new CameraObject("STE_LABEL_LprCamera", 1, "Lpr Camera", true);
            vm.cameras.push(lprCamera);

            // create context camera only if SharpSubType is not Its
            if (!isIts) {
                var contextCamera = new CameraObject("STE_LABEL_ContextCamera", 2, "Context Camera", false);
                vm.cameras.push(contextCamera);
            }
        }

        function CameraObject(name, id, tag, isLpr) {
            this.name = name;
            this.id = id;
            this.tag = tag;
            this.isLpr = isLpr;
        }       

        // Clean up to perform when the view is closed
        $scope.$on("$destroy", function () {
            $timeout.cancel(vm.timeout);
            defer.reject();

            if (vm.registeredFeedInfo3) {
                vm.registeredFeedInfo3(); // unregister handler
                vm.registeredFeedInfo3 = null;
            }

            if (vm.registeredWatch) {
                vm.registeredWatch(); // clear previous watch
                vm.registeredWatch = null;
            }

            if (vm.removeListener) {
                vm.removeListener(); // unregister handler
                vm.removeListener = null;
            }
        });


        var impossibleValue = -1000000;
        var focusAnalysisPoints = []; // points representing the improvement of focus relative to previous image
        var focusScaleY = 125.0; // start at full scale (don't forget it's a float number), downsize if data goes outside graph
        var bestFocusPoint = impossibleValue; // set to some very low impossible value
        var initialFocusPoint = impossibleValue; // set to some very low impossible value

        var img = $window.document.getElementById("imageVideo");

        // This is what will asyncronously be performed when the image 'src' property is changed:
        if (img)
            img.onload = onVideoCanvasImageLoaded;

        function selectCamera(selectedCamera) {

            portalService.disableLogout();            

            vm.form.$setDirty();

            img = $window.document.getElementById("imageVideo");

            if (img)
                img.onload = onVideoCanvasImageLoaded;

            if (selectedCamera === vm.selectedCamera)
                return; // reselected the same camera

            // Check if a camera is already chosen and alert user, if any
            if (vm.selectedCamera !== undefined && vm.selectedCamera.id > 0) {
                if (promptUserForTerminateOrCancel())
                    terminate(true); // terminate previous camera and turn to new selected camera
                else
                    return; // cancel the request
            }

            vm.isAlertShown = false;
            vm.isIrisFullyOpened = true; // to prevent display of the warning initially
            vm.selectedCamera = selectedCamera;
            vm.canUseFlash = selectedCamera.id === 2; // Only context camera can control flash.

            resetFocusGraph();

            if (selectedCamera.id === 0)
                return; // nothing to do

            if (img) {
                img.toFree = null;
                img.src = noImage; // initially indicate no feed is available
            }

            if (vm.registeredWatch)
                vm.registeredWatch(); // clear previous watch

            vm.registeredWatch = $scope.$watch("vm.currentExposure", function (value) {
                if (value != undefined)
                    updateFocusExposureIndex(value);
            });

            // Stop auto-exposure, allow temporarily override of exposure settings (keep a backup) and open iris to full.
            // Should be done in the back-end so that it can restore in case of problem
            portalService.beginZoomAndFocus(selectedCamera.tag)
                .then(function (value) {
                // The back-end can implement an auto-exposure, auto-flash-enabling
                // to initialize the exposure parameters.
                vm.currentExposure = value.data.ExposureLevel * 100;
                vm.flashEnabled = value.data.FlashEnabled;
                vm.isIrisFullyOpened = value.data.SetFocusExposureResult.IrisLevel === 1; // if iris level != 1.0, display a warning to the user

                // prevent user from changing page without a warning
                vm.removeListener = $rootScope.$on('$stateChangeStart', function (event, toState, toParams, fromState, fromParams, options) {
                    var canContinue = confirm($translate.instant("STE_SENTENCE_SettingsChangedContinueWithoutSaving"));
                    if (canContinue) {
                        event.preventDefault(); // prevent state transition until record is stoped
                        vm.removeListener();
                        portalService.endZoomAndFocus(vm.selectedCamera.tag)
                            .then(function () {
                                // Success handler
                            }, function (error) {
                                Console.log(error);
                                // Error handler
                            }).finally(function () {
                                portalService.enableLogout();
                                $state.go(toState, toParams); // will process the state transition even if call fails
                            });
                    }

                    event.preventDefault();
                });

                vm.registeredFeedInfo3 = $rootScope.$on("ImageFeedInfoDone", function (e, message) {
                    $rootScope.$apply(function () {

                        var cameraInfo = message;

                        // Update focus analysis data
                        addfocusScore(cameraInfo.FocusQuality);

                        drawFocusGraph();
                    });
                });

                // Query the feed corresponding to the chosen camera
                var isLprCamera = (selectedCamera.id === 1); // assuming first camera displayed is LPR... this is not very robust, but it works for now
                var request = new signalRService.FeedRequestOptions(1, isLprCamera, !isLprCamera);
                signalRService.initializeFeedHub(request);
            });
        }

        function addfocusScore(newValue) {
            //if (focusAnalysisPoints.length > 0 && focusAnalysisPoints[focusAnalysisPoints.length - 1] == newValue)
            //    return; // don't add identical points

            if (countDownBeforeFocusGraph > 0) {
                // If a command has been sent recently that caused a change in the
                // focus indicator, then ignore the first few received values because
                // they might have been calculated with the old conditions.
                countDownBeforeFocusGraph--;
                return;
            }

            if (newValue > bestFocusPoint)
                bestFocusPoint = newValue;
            if (initialFocusPoint === impossibleValue)
                initialFocusPoint = newValue;
            focusAnalysisPoints.push(newValue); // new point to add
            if (focusAnalysisPoints.length > 2000) // gather as much as we can (to fit on a screen). irrelevant data will be ignored while rendering
                focusAnalysisPoints.splice(0, 1); // make sure the maximum number of item fits in the view
        }

        function checkFocusGraphRescale(refY) {
            if (focusAnalysisPoints.length > 0) {
                var lastPoint = focusAnalysisPoints[focusAnalysisPoints.length - 1];
                if (((lastPoint - initialFocusPoint) * focusScaleY > refY) || ((lastPoint - initialFocusPoint) * focusScaleY < -refY)) {
                    // Recalculate min, max and center around initialPoint
                    var min = focusAnalysisPoints[0];
                    for (var index = 1; index < focusAnalysisPoints.length; index++) {
                        if (focusAnalysisPoints[index] < min)
                            min = focusAnalysisPoints[index];
                        initialFocusPoint = (min + bestFocusPoint) / 2; // take middle point
                    }
                    focusScaleY = (refY / Math.abs(lastPoint - initialFocusPoint)) * 0.85;
                }
            }
        }

        function drawFocusGraph() {
            var canvas = $window.document.getElementById("graphCanvas");

            if (canvas == null) // we are unit testing
                return;

            var parent = canvas.parentElement;

            // then set the internal size to match
            if (parent.clientWidth > 0)
                canvas.width = parent.clientWidth;
            canvas.height = 150;

            // Make sure we don't execute when canvas isn't supported
            if (canvas.getContext) {
                // Note: origin is top left.
                var ctx = canvas.getContext("2d");

                var refY = (canvas.height) / 2;

                // Clear canvas prior to draw on it
                ctx.clearRect(0, 0, canvas.width, canvas.height);

                // If last point entered is outside the graph, rescale it automatically
                checkFocusGraphRescale(refY);

                // Draw the best line (0 is on top, growing down)
                ctx.strokeStyle = "#57401B"; // color
                ctx.lineWidth = 3;
                ctx.beginPath();
                var y = refY - (bestFocusPoint - initialFocusPoint) * focusScaleY;
                if (y > canvas.height)
                    y = canvas.height;
                ctx.moveTo(canvas.width, y); // origin
                ctx.lineTo(0, y); // X-axis
                ctx.stroke();

                // Draw the points (0 is on top, growing down)
                if (focusAnalysisPoints.length > 0) {
                    ctx.strokeStyle = "#FF9900"; // color
                    ctx.lineWidth = 4;
                    ctx.beginPath();
                    var maxPoints = focusAnalysisPoints.length < canvas.width ? focusAnalysisPoints.length : canvas.width;
                    ctx.moveTo(canvas.width - maxPoints, refY - (focusAnalysisPoints[0] - initialFocusPoint) * focusScaleY); // first point
                    for (var index = 1; index < maxPoints; index++) {
                        ctx.lineTo(canvas.width - maxPoints + index, refY - (focusAnalysisPoints[index] - initialFocusPoint) * focusScaleY);
                    }
                    ctx.stroke();

                    // Make room at the left of graph (end) to emphasize the drawing still occurs when the value don't change
                    if (maxPoints >= canvas.width)
                        focusAnalysisPoints.splice(0, focusAnalysisPoints.length - maxPoints + 30);
                }

                // Draw the axis (not affected by scale)
                ctx.strokeStyle = "#333333"; // color
                ctx.lineWidth = 4;
                ctx.beginPath();
                ctx.moveTo(canvas.width, canvas.height); // origin
                ctx.lineTo(0, canvas.height); // X-axis
                ctx.moveTo(canvas.width, canvas.height); // origin
                ctx.lineTo(canvas.width, 0); // Y-axis
                ctx.stroke();
            }
        }

        function resetFocusGraph() {
            focusAnalysisPoints = []; // no more points
            bestFocusPoint = impossibleValue;
            initialFocusPoint = impossibleValue;
            focusScaleY = 125.0;

            // Stop updating the graph for some time,
            // otherwise, we might include points with the old conditions in the future graph.
            countDownBeforeFocusGraph = 10; 
            // the value 10 should be enough in most situations (except maybe for extreme iris changes)
            // and it should not produce an unaceptable delay before graph restarts.
            // Obtaining more accurate condition to restart the focus graph would 
            // require more complicated feedback mechanisme to know if the previous
            // commands have really been applied.

            drawFocusGraph();
        }

        function resetExposure() {
            portalService.resetFocusExposure(vm.selectedCamera.tag)
                .then(function (value) {
                    vm.currentExposure = value.data.ExposureLevel * 100;
                    vm.flashEnabled = value.data.FlashEnabled;
                    vm.isIrisFullyOpened = value.data.SetFocusExposureResult.IrisLevel === 1; // if iris level != 1.0, display a warning to the user
                });
        }

        // called when "+" button is clicked, call to update the server is immediate
        function raiseExposure() {
            if (vm.currentExposure < 100) {
                vm.currentExposure++;
            }
        }

        // called when "-" button is clicked, call to update the server is immediate
        function lowerExposure() {
            if (vm.currentExposure > 0) {
                vm.currentExposure--;
            }
        }

        function updateFlashEnabled() {
            updateFocusExposureIndex(vm.currentExposure);
        }

        function updateFocusExposureIndex(index) {

            $log.debug("Setting focus Exposure index : " + index);
            portalService.setFocusExposureLevel(vm.selectedCamera.tag, index / 100, vm.flashEnabled)
                .then(function(value) {
                    vm.isIrisFullyOpened = value.data.IrisLevel === 1; // if iris level != 1.0, display a warning to the user
                });
            resetFocusGraph(); // Because exposure changes the value of focus indicator.
        }

        function terminate(cameraSwich) {
            vm.isTerminating = true;

            if (vm.removeListener) {
                vm.removeListener();
                vm.removeListener = null;
            }

            // Stop the LPR info feed.
            signalRService.stopFeed();

            if (vm.registeredFeedInfo3) {
                vm.registeredFeedInfo3(); // unregister handler
                vm.registeredFeedInfo3 = null;
            }

            if (vm.registeredWatch) {
                vm.registeredWatch(); // clear previous watch
                vm.registeredWatch = null;
            }

            // Clear the value so that it won't try to set a value when selecting another camera
            vm.currentExposure = null;

            // Restore auto-exposure to previous values and restore auto-iris
            portalService.endZoomAndFocus(vm.selectedCamera.tag)
                .then(function () {
                    vm.isTerminating = false;

                    if (!cameraSwich) {
                        // when terminating on destroy, there is no form
                        terminateCompleted();
                    }
                });            
        }

        function terminateCompleted() {
            showAlert("success", $translate.instant("STE_SENTENCE_RegularOperationsResumed"));

            vm.isTerminating = false;

            // Clear combobox
            vm.selectedCamera = vm.cameras[0];

            // Clear graph
            resetFocusGraph();

            // Reset form if any
            if (vm.form)
                vm.form.$setPristine();
        }

        function promptUserForTerminateOrCancel() {

            return true; // always allow user to pick another camera by terminating the current one...

            //var ok = $window.confirm("Click OK if you want to terminate current zoom and focus operation.");
            //return ok; // don't allow selection of another camera while terminate is not done
        }

        // type can be: success, info, warning or danger
        // msg is a string
        function showAlert(type, msg) {
            vm.isAlertShown = true;
            vm.alert = { type: type, msg: msg }
            $timeout(function () {
                vm.isAlertShown = false;
            }, 5000);
        }
    }
})();
