﻿(function () {
    "use strict";

    angular
        .module("portalApp.controllers")
        .controller("CameraFeedsController", CameraFeedsController);

    CameraFeedsController.$inject = ["$scope", "$rootScope", "portalService", "signalRService", "$window", "$uibModal", "backendServerUrl", "$translate", "$q", "$state", "$timeout"];

    function CameraFeedsController($scope, $rootScope, portalService, signalRService, $window, $uibModal, backendServerUrl, $translate, $q, $state, $timeout) {
        var vm = this;
        vm.title = "CameraFeedsController";
        vm.showCrosshair = true; // in context camera
        vm.showBoundingBox = true; // in LPR camera
        vm.showROI = true; // in LPR camera
        vm.cameras = [];
        vm.lastPlateNumber = "-";
        vm.lastPlateState = "-";
        vm.candidate = "-";
        vm.nbReads = 0;
        vm.analytics = [];
        vm.resetReadsCount = resetReadsCount;
        vm.getCameraObject = getCameraObject;
        vm.selectCameraGroup = selectCameraGroup;
        vm.noImage = "images/noVideo.png";

        vm.selectedCameraGroup = 1; // first camera group
        vm.cameraGroups = ["STE_LABEL_NoCamera", "STE_LABEL_FirstCameraGroup"];
        vm.getSelectedCameraGroupName = getSelectedCameraGroupName;

        vm.videoActivity = undefined;
        vm.videoActivityChecked = false;
        vm.videoErrorConnection = false;

        // variables for the image capture dialog:
        vm.recordingDestination = "";
        vm.recordingDuration = 15;
        vm.recordingTopic = "saveAllImages";
        vm.maxMinutes = 0; // will be overridden in saveAllImages(...)
        vm.saveAllImages = saveAllImages;
        vm.close = close;
        vm.nbImagesGrabbed = 0;
        vm.recording = false;
        vm.recordDisabled = false;
        vm.setConfiguration = setConfiguration;
        vm.startStopRecording = startStopRecording;
        vm.timeouts = [];
        vm.userConfig = {}; // Data binding for the video feed directive

        saveAllImages(true); // sets 'vm.maxMinutes'

        $rootScope.isLoading = true;
        $rootScope.connectionLost = false;
        $rootScope.reconnecting = false;
        $rootScope.activated = false;
        var defer = $q.defer();

        $scope.$on("$destroy", function () {
            $timeout.cancel(vm.timeout);
            defer.reject();
            vm.unregisterFeed3();
            vm.unregisterDownload();
            vm.unregisterConnectionLost();
            clearInterval(vm.timer);

            // Stop the video info feed.
            signalRService.stopFeed();
        });

        activate().then(function () {
            $rootScope.isLoading = false;
            $rootScope.connectionLost = false;
            $rootScope.reconnecting = false;
            $rootScope.activated = true;

            // By default, stream all information
            var request = new signalRService.FeedRequestOptions(3, true, vm.getCameraObject("context") !== undefined);
            signalRService.initializeFeedHub(request);
        });

        function activate() {
            var request = function () {
                if ($rootScope.reconnecting)
                    $rootScope.activated = true;

                var promises = [
                    portalService.getUserConfig(),
                    portalService.getRegionOfInterest("Lpr Camera"),
                    portalService.getProperties()
                ];

                return $q.all(promises).then(
                    function success(responses) {
                        $rootScope.isLoading = false;
                        $rootScope.connectionLost = false;
                        $rootScope.reconnecting = false;
                        $rootScope.activated = true;

                        vm.userConfig = responses[0].data;
                        vm.userConfig.regionOfInterest = responses[1].data;
                        vm.setConfiguration(responses[2].data.IsIts);

                        defer.resolve();
                    },
                    function error() {
                        $rootScope.connectionLost = true;
                        $rootScope.isLoading = false;
                        // wait 5 seconds then retry
                        vm.timeout = $timeout(function () {
                            $rootScope.reconnecting = true;
                            request();
                        }, 5000);
                    }
                );
            }

            request();

            return defer.promise;
        }

        function saveAllImages(saveAll) {
            // 99 minutes for "saveAllImages", 2880 minutes (48 hours) for "saveBestReadsOnly"
            vm.maxMinutes = saveAll ? 99 : 2880;
        }

        vm.unregisterFeed3 = $rootScope.$on("NewImageFeedInfo", function (e, message) {
            $rootScope.$apply(function () {

                var imageInfo = message;

                if (imageInfo === null)
                    return;

                vm.videoErrorConnection = false;

                if (vm.selectedCameraGroup > 0) { // when hiding the cameras, don't update the video, but stil update reads information                    

                    var camera = imageInfo.IsLprCamera ? vm.getCameraObject("lpr") : vm.getCameraObject("context");
                    var exposure = parseExposure(imageInfo, imageInfo.IsLprCamera ? "Lpr Exposure" : "Context Exposure");
                    camera.exposureTime = exposure.exposureTime;
                    camera.exposureGain = exposure.gain;
                    camera.exposureIris = exposure.iris;
                    camera.exposureFlash = exposure.flash;
                    camera.name = imageInfo.CameraName;

                    if (imageInfo.FrameRatePerSecond || imageInfo.FrameRatePerSecond === 0.0)
                        camera.frameRate = imageInfo.FrameRatePerSecond;

                    if (imageInfo.AverageFrameRatePerSecond || imageInfo.AverageFrameRatePerSecond === 0.0)
                        camera.avgFrameRate = imageInfo.AverageFrameRatePerSecond;

                    if (imageInfo.IsLprCamera) {
                        // Update the globally saved statistics about last best read
                        vm.lastPlateNumber = imageInfo.BestRead;
                        vm.nbReads = imageInfo.BestReadCount;
                        if (imageInfo.LprRead)
                            vm.candidate = imageInfo.LprRead; // update last read candidate (keep it displayed until replaced)

                        // Update last read panel info when a "best" Read is generated
                        if (imageInfo.FeedType === 2) {
                            vm.lastPlateState = imageInfo.State;
                            updateAnalytics(imageInfo.CustomFields);
                        }

                        // Blink the led if a plate was found (even if a "best" Read was not issued)
                        var ledName = "led-green";
                        var element = $("#feedActivityLed");
                        if (imageInfo.LprRead) { // read or best read
                            element.addClass(ledName);
                            setTimeout(function () {
                                element.removeClass(ledName);
                            }, 300);
                        }
                    }
                }
            });
        });

        vm.unregisterDownload = $rootScope.$on("DownloadProgression", function (e, token, count, completing, completed) {
            $rootScope.$apply(function () {

                if (token === vm.recordingDestination) { // check if this is our client
                    vm.nbImagesGrabbed = count;

                    if (completed) {
                        vm.recording = false;
                        vm.recordDisabled = false;
                        vm.removeListener();
                        portalService.enableLogout();
                    }
                }
            });
        });

        vm.unregisterConnectionLost = $rootScope.$on("LostFeedConnection", function (e, message) {
            $rootScope.$apply(function () {
                vm.videoErrorConnection = true;
            });
        });

        function setConfiguration(isIts) {
            // create lpr camera object
            var lprCamera = new CameraObject("Lpr Camera", "STE_LABEL_LprCamera", "lpr");
            vm.cameras.push(lprCamera);

            // create context camera only if SharpSubType is not Its
            if (!isIts) {
                var contextCamera = new CameraObject("Context Camera", "STE_LABEL_ContextCamera", "context");
                vm.cameras.push(contextCamera);
            }
        }

        function CameraObject(name, displayName, cameraType) {
            this.cameraName = name;
            this.displayName = displayName;
            this.frameRate = "-";
            this.avgFrameRate = "-";
            this.resolution = "-";
            this.exposureTime = "-";
            this.exposureGain = "-";
            this.exposureIris = "-";
            this.exposureFlash = "-";
            this.cameraType = cameraType;
            this.isLprCamera = function () {
                return this.cameraType.toLowerCase() === "lpr";
            };
        }

        function getCameraObject(cameraType) {
            var found = vm.cameras.filter(function (camera) { return camera.cameraType === cameraType });
            if (found.length > 0)
                return found[0];
            return undefined;
        }

        function parseExposure(imageInfo, tag) {
            var exposureTime = "-";
            var gain = "-";
            var iris = "-";
            var flash = "-";

            if (imageInfo.CustomFields && imageInfo.CustomFields.hasOwnProperty(tag)) {
                // See encoding code in PlateReader (EvoChannelAdapter.cs)
                // The format is: "(xxx) yyy.y ms vvv.v dB Iris:www.w% Flash:zzz%"
                //                     1       2        3  4           5        
                var full = imageInfo.CustomFields[tag];
                var index1 = full.indexOf(") ");
                var index2 = full.indexOf("ms ");
                var index3 = full.indexOf("dB ");
                var index4 = full.indexOf("Iris:");
                var index5 = full.indexOf("Flash:");
                if (index1 > 0 &&
                    index2 > 0 &&
                    index3 > 0 &&
                    index4 > 0 &&
                    index5 > 0 &&
                    index1 < index2 &&
                    index2 < index3 &&
                    index3 < index4 &&
                    index4 < index5 &&
                    index5 < full.length - 1) {
                    exposureTime = full.substring(index1 + 2, index2 + 2);
                    gain = full.substring(index2 + 2, index3 + 2);
                    iris = full.substring(index4 + 5, index5 - 1);
                    flash = full.substring(index5 + 6);
                }
            }

            return {
                exposureTime: exposureTime,
                gain: gain,
                iris: iris,
                flash: flash
            };
        }

        function startStopRecording() {
            if (!vm.recording)
                vm.startRecording();
            else
                vm.stopRecording();
        }

        vm.startRecording = function () {

            vm.recording = false;
            vm.nbImagesGrabbed = 0;

            // Display a dialog to choose parameters, start recording and monitor the progression
            // while allowing the user to cancel the operation.
            $scope.modalInstance = $uibModal.open({
                templateUrl: "imageCapture.html",
                scope: $scope,
                size: "md",
                backdrop: 'static'
            });

            // When the dialog is closed, do the following:
            $scope.close = function (cancelled) {

                $scope.modalInstance.close();

                if (!cancelled) { // check that dialog was not cancelled

                    var cameraNames = [];
                    angular.forEach(vm.cameras, function (camera) {
                        cameraNames.push(camera.cameraName);
                    });

                    portalService.registerImageCapture(cameraNames, vm.recordingTopic, vm.recordingDuration)
                        .then(function (response) {
                            // success
                            vm.recordingDestination = response.data; // return token to start download
                            vm.recording = true;

                                portalService.disableLogout();

                            // prevent user from changing page without a warning
                            vm.removeListener = $rootScope.$on('$stateChangeStart', function (event, toState, toParams, fromState, fromParams, options) {
                                var canContinue = confirm($translate.instant("STE_SENTENCE_RECORDOPERATIONINPROGRESS"));
                                if (canContinue) {
                                    event.preventDefault(); // prevent state transition until record is stoped
                                    vm.removeListener();
                                    portalService.stopImageCapture(vm.recordingDestination)
                                        .then(function () {
                                            // Success handler
                                        }, function () {
                                            // Error handler
                                        }).finally(function () {
                                            portalService.enableLogout();
                                            $state.go(toState, toParams); // will process the state transition even if call fails
                                        });
                                }

                                event.preventDefault();
                            });

                            // Request to change the current page location.
                            // Browser will intercept it and download the response as a file on disk instead.
                            var downloadUrl = backendServerUrl + "/portal/downloadCapturedImages?Bearer=" + $window.sessionStorage.accessToken + "&token=" + vm.recordingDestination;
                            $window.location.href = downloadUrl;
                        }
                        , function () {
                            // failure
                            // Give some feed-back
                            $("#recordButton").addClass('shake').delay(1500).queue(function () {
                                $(this).removeClass('shake');
                                $(this).dequeue();
                            });
                        }
                        );
                }
            };
        };

        // ask server to abort pending recording
        vm.stopRecording = function () {
            vm.recording = false;
            vm.recordDisabled = true;
            vm.removeListener();
            portalService.stopImageCapture(vm.recordingDestination)
                .then(function () {
                    // carry on
                }, function () {
                    vm.recordDisabled = false;
                }).finally(function () {
                    portalService.enableLogout();
                });
        };

        function selectCameraGroup(selectedCameraGroup) {
            vm.selectedCameraGroup = selectedCameraGroup;
            var hasContext = vm.getCameraObject("context") !== undefined;

            // Update feed less often, since we don't need all the reads
            if (selectedCameraGroup === 0) {
                signalRService.reduceFeed(hasContext);
                vm.recordDisabled = true;
            } else {

                signalRService.startFeed(hasContext);
                vm.recordDisabled = false;
            }
        };

        function getSelectedCameraGroupName() {
            return vm.cameraGroups[vm.selectedCameraGroup];
        }

        function resetReadsCount() {
            // Send a command to reset the count in the server
            signalRService.resetCounter();
            vm.nbReads = 0;
        };

        function updateAnalytics(customFields) {
            // first clear all values
            for (var i = 0; i < vm.analytics.length; i++) {
                vm.analytics[i].value = "-";
            }
            // then put values from customFields
            for (var c in customFields) {
                if (customFields.hasOwnProperty(c)) {
                    var existingAnalytic = null;
                    for (var j = 0; j < vm.analytics.length; j++)
                        if (vm.analytics[j].analytic === c) {
                            existingAnalytic = vm.analytics[j];
                            break;
                        }
                    if (existingAnalytic)
                        existingAnalytic.value = customFields[c]; // replace value
                    else
                        vm.analytics.push({ analytic: c, value: customFields[c] }); // add a new value
                }
            }
        }
    }
})();
