﻿//
// Handles login request to the back-end and UI interaction
//

(function () {
    "use strict";

    angular
        .module("portalApp.controllers")
        .controller("RecoveryController", RecoveryController);

    RecoveryController.$inject = ["$state", "portalService", "focusFactory", "$rootScope", "$timeout", "$window", "signalRService", "$scope", "$translate"];

    function RecoveryController($state, portalService, focusFactory, $rootScope, $timeout, $window, signalRService, $scope, $translate) {
        var vm = this;
        vm.title = "RecoveryController";
        vm.isBusy = false;
        vm.passwordStrength = { score: 0 };
        vm.checkAccessCode = checkAccessCode;
        vm.setRecoveryMode = setRecoveryMode;
        vm.cancel = cancel;
        vm.resetPassword = resetPassword;
        vm.showAlert = showAlert;
        vm.redirect = redirect;
        vm.newPassword1 = undefined;
        vm.newPassword2 = undefined;
        vm.rdpPassword = "";
        vm.plate = "";
        vm.recoveryPlate = $window.sessionStorage.recoveryPlate;
        vm.isButton = true;
        vm.loginClass = 'show-opacity';
        $rootScope.bodyClass = "login";

        ///////////////////////////////////////////////   

        $scope.$on("$destroy", function () {

            // Unregister events
            if (vm.unregisterRecoveryPlate !== undefined)
                vm.unregisterRecoveryPlate();

            if (vm.unregisterRecoveryButton !== undefined)
                vm.unregisterRecoveryButton();

            if (vm.unregisterIsButton !== undefined)
                vm.unregisterIsButton();

            if (vm.unregisterWaitingRemaining !== undefined)
                vm.unregisterWaitingRemaining();

            if (vm.unregisterTimerExpired !== undefined)
                vm.unregisterTimerExpired();

            //signalRService.stopConnections();
        });

        // Register events that will bring the user to the page for setting the new password.
        // ONlY listen to events when in the proper state.
        if ($state.current.name === "recoveryPlate") {
            vm.unregisterRecoveryPlate = $rootScope.$on("RecoveryBestPlateReceived", function (e, message) {
                $rootScope.$apply(function() {
                    vm.showAlert('success', $translate.instant("STE_SENTENCE_PlateNumberRead"), true, function () {
                        $state.go("recoveryPassword"); // this will create a brand new controller
                    });
                });
            });
        }
        if ($state.current.name === "recoveryButton") {
            vm.unregisterRecoveryButton = $rootScope.$on("RecoveryResetButtonTriggered", function (e, message) {
                $rootScope.$apply(function() {
                    vm.showAlert('success', $translate.instant("STE_SENTENCE_ResetButtonPressed"), true, function () {
                        $state.go("recoveryPassword"); // this will create a brand new controller
                    });
                });
            });
        }

        vm.unregisterWaitingTimeRemaining = $rootScope.$on("WaitingTimeRemaining", function (e, message) {
            $rootScope.$apply(function() {
                vm.timeRemainingWaiting = convertTimeSpanStringToUi(message);
            });
        });

        vm.unregisterTimerExpired = $rootScope.$on("TimerExpired", function (e, message) {
            $rootScope.$apply(function () {
                vm.showAlert('warning', $translate.instant("STE_SENTENCE_CouldNotCompleteInTime"), true, function () {
                    redirect(); // go back to login page in all cases
                });
            });
        });

        // When initializing controller for one of the wait forms (button or plate), must be able to receive notifications
        if ($state.current.name === "recoveryButton" || $state.current.name === "recoveryPlate") {
            signalRService.initializeRecoveryHub();
        }
        else {
            signalRService.stopConnections();
        }

        vm.unregisterIsButton = $scope.$watch("vm.isButton",
            function(newVal, oldVal) {
                if (oldVal && !newVal) {
                    // Set focus to plate number field when radio button is chosen
                    focusFactory.doFocus("plate");
                }
            });

        function convertTimeSpanStringToUi(timeSpan) {
            // input format is: "hh:mm:ss.ddddddddd"
            // First cut decimals
            var decimals = timeSpan.split(".");
            
            // then separate every components
            var parts = decimals[0].split(":");

            return parts[0] + "h " + parts[1] + "m " + parts[2] + "s";
        }

        function checkAccessCode() {
            vm.error = undefined; // clear previous error if any

            // Start remote query
            vm.isBusy = true;
            portalService.checkAccessCode(vm.rdpPassword)
                .then(function (response) {
                    // success
                    vm.error = undefined;
                    // success, redirect to setup page (second authentication)
                    $window.sessionStorage.recoveryToken = response.data.recoveryToken;
                    $state.go("recoverySetup"); // this will create a brand new controller

                }, function (response) {
                    //failure
                    focusFactory.doFocus("password");
                    vm.error = response;
                }).finally(function () {
                    vm.isBusy = false;
                    vm.rdpPassword = ""; //reset password box                    
                });
        }

        function setRecoveryMode() {
            vm.error = undefined; // clear previous error if any

            // Start remote query
            vm.isBusy = true;
            if (vm.isButton) {
                // call plate reader to activate the selected recovery mode
                portalService.recoveryButton()
                    .then(function (response) {
                        // success
                        vm.error = undefined;
                        $state.go("recoveryButton"); // this will create a brand new controller
                    }, function (response) {
                        //failure
                        vm.error = response;
                    }).finally(function() {
                        vm.isBusy = false;
                    });
            } else {
                vm.plate = vm.plate.toUpperCase();
                // call plate reader to activate the selected recovery mode
                portalService.recoveryPlate(vm.plate)
                    .then(function(response) {
                        // success
                        vm.error = undefined;
                        $window.sessionStorage.recoveryPlate = vm.plate;
                        $state.go("recoveryPlate"); // this will create a brand new controller
                    }, function (response) {
                        //failure
                        vm.error = response;
                    }).finally(function() {
                        vm.isBusy = false; 
                    });
            }
        }

        function resetPassword() {

            var result = zxcvbn(vm.newPassword2);

            // Make sure the password meets the minimum criteria before submitting as the server can't do much on a hashed password.
            if (result.score < 3) {
                vm.showAlert('danger', $translate.instant("STE_SENTENCE_CannotSetNewPassword"));
                return;
            }

            vm.isBusy = true;
            // then save if all is Ok
            portalService.resetNewPassword(vm.newPassword2)
                .then(function (response) {
                    // success, clear token and go back to login
                    vm.showAlert('success', $translate.instant("STE_SENTENCE_PasswordChanged"), true, vm.redirect);
                    // no need to reset is busy, the browser will be redirected to the login page                    
                }, function () {
                    // failed
                    vm.showAlert('danger', $translate.instant("STE_SENTENCE_CannotSetNewPassword"));
                    vm.isBusy = false;
                });
        }

        // clear both recovery session objects and go to login page
        function redirect() {
            delete $window.sessionStorage.recoveryToken;
            delete $window.sessionStorage.recoveryPlate;
            $state.go("login");
        }

        // type can be: success, info, warning or danger
        // msg is a string
        function showAlert(type, msg, timeout, continueWith) {
            vm.alert = { type: type, msg: msg }
            vm.show = true;
            if (timeout) {
                $timeout(function () {
                    vm.show = false;

                    if (!!continueWith && typeof continueWith == "function") {
                        continueWith();
                    }

                }, 5000);
            }
        }

        function cancel() {
            portalService.cancelReset()
                .then(function () {
                    redirect();
                }, function (response) {
                    // special case when PlateReader is down, we get 404 so we can redirect because the PlateReader will not be in recovery mode when restarted
                    if (response.status === 404) {
                        redirect();
                    } else {
                        vm.showAlert('danger', $translate.instant("STE_SENTENCE_ConnectionLost"));
                    }
                });
        }
    }
})();

