﻿/******************************* HOW TO USE THIS DIRECTIVE *******************************/
// - Add ipv4-and-subnet-mask-validator as an attribute to an element.
// - This directive uses 4 parameters that you also need to add as attributes to an element
//   - is-subnet-mask: Indicates that the field that has not been modified is the subnet mask.
//   - subnet-mask: Use this parameter when is-subnet-mask == true
//   - ip-address: Use this parameter when is-subnet-mask == false
//   - input-error: Error related to this directive, so you need to pass the directive's name as value to this attribute

(function () {
    "use strict";

    angular.module("portalApp.directives")
        .directive("ipv4AndSubnetMaskValidator", function () {
            return {
                require: "ngModel",
                restrict: "A",
                link: function (scope, element, attributes, controller) {
                    controller.$validators.ipv4AndSubnetMaskValidator = function (updatedField) {
                        // Clear previous IP and Subnet combination errors to allow that a change of one of the two fields can "revalidate" the combination
                        clearError();

                        return validateIPAddressAndSubnetMaskCombination(updatedField);
                    };

                    function clearError() {
                        var form = controller.$$parentForm;

                        angular.forEach(form, function (ctrl, formName) {
                            // Ignore angular related fields and functions
                            if (formName.indexOf("$") !== 0) {
                                angular.forEach(ctrl.$error, function (errorValue, errorName) {
                                    if (attributes.inputError == null || attributes.inputError == undefined) {
                                        return;
                                    }

                                    // Only clear the error related to this directive
                                    if (errorName === attributes.inputError) {
                                        ctrl.$setValidity(errorName, true);
                                    }
                                });
                            }
                        });
                    }

                    function toBinaryString(str) {
                        return ("00000000" + str.toString(2)).slice(-8);
                    }

                    function validateIPAddressAndSubnetMaskCombination(updatedField) {
                        var ip, subnetMask;
                        if (attributes.isSubnetMask === "true") {
                            ip = updatedField;
                            subnetMask = attributes.subnetMask;
                        } else {
                            ip = attributes.ipAddress;
                            subnetMask = updatedField;
                        }

                        var ipParts = ip.split(".");
                        var maskParts = subnetMask.split(".");

                        // Store IP address in variables to improve readability
                        var i1 = toBinaryString(parseInt(ipParts[0]));
                        var i2 = toBinaryString(parseInt(ipParts[1]));
                        var i3 = toBinaryString(parseInt(ipParts[2]));
                        var i4 = toBinaryString(parseInt(ipParts[3]));

                        // IP & Mask ==> Network address
                        var n1 = toBinaryString(parseInt(i1, 2) & parseInt(maskParts[0]));
                        var n2 = toBinaryString(parseInt(i2, 2) & parseInt(maskParts[1]));
                        var n3 = toBinaryString(parseInt(i3, 2) & parseInt(maskParts[2]));
                        var n4 = toBinaryString(parseInt(i4, 2) & parseInt(maskParts[3]));

                        // Store inverted mask in variables to improve readability
                        var m1 = toBinaryString(parseInt(maskParts[0])).replace(/\d/g, function (c) { return c ^ 1 });
                        var m2 = toBinaryString(parseInt(maskParts[1])).replace(/\d/g, function (c) { return c ^ 1 });
                        var m3 = toBinaryString(parseInt(maskParts[2])).replace(/\d/g, function (c) { return c ^ 1 });
                        var m4 = toBinaryString(parseInt(maskParts[3])).replace(/\d/g, function (c) { return c ^ 1 });

                        // Network address | ~Mask ==> Broadcast address
                        var b1 = toBinaryString(parseInt(n1, 2) | parseInt(m1, 2));
                        var b2 = toBinaryString(parseInt(n2, 2) | parseInt(m2, 2));
                        var b3 = toBinaryString(parseInt(n3, 2) | parseInt(m3, 2));
                        var b4 = toBinaryString(parseInt(n4, 2) | parseInt(m4, 2));

                        var ipAddress = i1 + "." + i2 + "." + i3 + "." + i4;
                        var networkAddress = n1 + "." + n2 + "." + n3 + "." + n4;
                        var broadcastAddress = b1 + "." + b2 + "." + b3 + "." + b4;

                        return ipAddress !== networkAddress && ipAddress !== broadcastAddress;
                    }
                }
            };
        });
})();