﻿"use strict";

describe("Camera Feeds controller", function () {
        var controller, portalService, scope, $state, signalRService, initializeFeedHubSpy, rootScope, $uibModal, $window, $httpBackend, window, $translate, sandbox, $injector, mockFactory;
        var backendServerUrl = "http://localhost:8080";
        var backendLoggerUrl = "http://localhost:88";
        var dtoRegionOfInterest = { data: { Coordinates: [1, 2, 3, 4, 5, 6] } };
        var dtoProperties = { data: { IsIts: false } };
        var dtoUserConfig = { data: { } };

        beforeEach(function() {
            module("portalApp.controllers",
                function($provide) {
                    $provide.value("backendServerUrl", backendServerUrl);
                    $provide.value("backendLoggerUrl", backendLoggerUrl);
                });
            module("portalApp.services");
            module("ui.bootstrap");
            module("pascalprecht.translate"); // for $translate
            module("portalApp.testing"); // load the module, otherwise: [$injector:unpr] Unknown provider: mockFactoryProvider <- mockFactory

            sandbox = sinon.sandbox.create();

            inject(function(_$injector_, $rootScope, _$translate_) {
                $injector = _$injector_;
                scope = $rootScope.$new();
                signalRService = $injector.get("signalRService");
                portalService = $injector.get("portalService");
                $state = $injector.get("$state");
                $uibModal = $injector.get("$uibModal");
                $window = $injector.get("$window");
                $httpBackend = $injector.get("$httpBackend");
                $translate = _$translate_;
                mockFactory = $injector.get("mockFactory");
            });

            mockFactory.setSandbox(sandbox);
            mockFactory.stubGetRegionOfInterest(dtoRegionOfInterest);
            mockFactory.stubGetProperties(dtoProperties);
            mockFactory.stubGetUserConfig(dtoUserConfig);
        });

        afterEach(function () {
            sandbox.restore();
        });

    it("should be created successfully", function () {
        inject(function ($injector, $controller, $rootScope) {
            initializeFeedHubSpy = sandbox.stub(signalRService, "initializeFeedHub");
            controller = $controller("CameraFeedsController", { $scope: scope, portalService: portalService, signalRService: signalRService });
            $rootScope.$apply();
        });

        expect(controller).to.be.an('object');
        expect(controller.title).to.equal("CameraFeedsController");
        expect($state.current.name).to.equal(""); // no state should be loaded
        sinon.assert.calledOnce(initializeFeedHubSpy);
    });

    it("should handle event when rootscope emits NewImageFeedInfo", function () {
        var imageFeedInfos =
        {
            ImageString: "test1234",
            FrameRatePerSecond: 20,
            AverageFrameRatePerSecond: 23,
            IsLprCamera: true,
            ImageCount: 1,
            CameraName: "My Camera"
        };

        inject(function ($injector, $controller, $rootScope, $window, $uibModal) {
            initializeFeedHubSpy = sandbox.stub(signalRService, "initializeFeedHub");
            controller = $controller("CameraFeedsController", { $scope: scope, portalService: portalService, signalRService: signalRService, $window: $window, $uibModal: $uibModal });
            rootScope = $rootScope;            
            $rootScope.$apply();
        });

        rootScope.$emit("NewImageFeedInfo", imageFeedInfos);
        
        sinon.assert.calledOnce(initializeFeedHubSpy);

        expect(controller.cameras[0].imgSrc).should.be.an('object');
        expect(controller.cameras[0].frameRate).to.equal(imageFeedInfos.FrameRatePerSecond);
    });

    it("should handle DownloadProgression when starting recording", function () {

        var imageFeedInfosStart =
        {
            ImageString: "test1234",
            FrameRatePerSecond: 20,
            AverageFrameRatePerSecond: 23,
            IsLprCamera: true,
            ImageCount: 1,
            CameraName: "My Camera",
            ImageChunks:2
        };

        var imageFeedInfosChunk1 =
        {
            ImageString: "test1234",
            FrameRatePerSecond: 20,
            AverageFrameRatePerSecond: 23,
            IsLprCamera: true,
            ImageCount: 1,
            CameraName: "My Camera",
            Chunk: 1
        };

        var imageFeedInfosChunk2 =
        {
            ImageString: "test1234",
            FrameRatePerSecond: 20,
            AverageFrameRatePerSecond: 23,
            IsLprCamera: true,
            ImageCount: 1,
            CameraName: "My Camera",
            Chunk: 2
        };

        var imageFeedInfosDone =
        {
            ImageString: "test1234",
            FrameRatePerSecond: 20,
            AverageFrameRatePerSecond: 23,
            IsLprCamera: true,
            ImageCount: 1,
            CameraName: "My Camera",
            ImageChunks: 2,
            ThumbnailPoints: null
        };

        var token = "ImageCapture_2016_04_08-16_50_52.1.zip";

        inject(function ($injector, $controller, $rootScope, $uibModal) {
            sandbox.stub(signalRService, "initializeFeedHub");
            sandbox.stub(portalService, "stopImageCapture");
            var deferredData = $injector.get("$q").defer();
            deferredData.resolve({ data: token });
            sandbox.stub(portalService, "registerImageCapture").returns(deferredData.promise);
            sandbox.stub($uibModal, "open").returns({ close: function() {} });

            window = {
                location: { href: "" },
                document: {
                    getElementById: function () {
                        return { getContext: null };
                    }
                },
                sessionStorage: { accessToken: "Bonjour", refreshToken: "Allo" }
            };

            sinon.stub(window.document, "getElementById").returns(null); // for draw contextVideo and drawLprVideo

            controller = $controller("CameraFeedsController", { $scope: scope, portalService: portalService, signalRService: signalRService, $window: window, $uibModal: $uibModal });

            rootScope = $rootScope;
            $rootScope.$apply();
        });

        // Start recording
        controller.nbImagesGrabbed = 100;
        controller.startRecording();
        scope.close(false); // force closing the parameters dialog
        rootScope.$digest(); // evaluate the promise used in startRecording()
        expect(controller.recordingDestination).to.equal(token);
        expect(controller.recording).to.equal(true); // started recording

        // Receive first progression message
        rootScope.$emit("DownloadProgression", token, 0, false, false);
        expect(controller.nbImagesGrabbed).to.equal(0);
        expect(controller.recording).to.equal(true); // still true

        // Receive second progression message
        rootScope.$emit("NewImageFeedInfo", imageFeedInfosStart);
        rootScope.$emit("NewImageChunk", imageFeedInfosChunk1);
        rootScope.$emit("NewImageChunk", imageFeedInfosChunk2);
        rootScope.$emit("ImageFeedInfoDone", imageFeedInfosDone);
        rootScope.$emit("DownloadProgression", token, 1, false, false);
        expect(controller.nbImagesGrabbed).to.equal(1);
        expect(controller.recording).to.equal(true); // still true

        // Notified the transfer is about to terminate
        rootScope.$emit("NewImageFeedInfo", imageFeedInfosStart);
        rootScope.$emit("NewImageChunk", imageFeedInfosChunk1);
        rootScope.$emit("NewImageChunk", imageFeedInfosChunk2);
        rootScope.$emit("ImageFeedInfoDone", imageFeedInfosDone);
        rootScope.$emit("DownloadProgression", token, 2, true, false); //for now, we don't use that 'completing' flag
        expect(controller.nbImagesGrabbed).to.equal(2);
        expect(controller.recording).to.equal(true); // still true

        // Notified the transfer is terminated
        $httpBackend.expectPOST(backendServerUrl + "/portal/stopImageCapture/" + token).respond(200);
        rootScope.$emit("DownloadProgression", token, 2, true, true);
        expect(controller.recording).to.equal(false); // stopped recording
    });

    it("should handle improper starting recording", function () {

        inject(function ($injector, $controller, $rootScope, $window, $uibModal) {
            sinon.stub(signalRService, "initializeFeedHub");
            var deferredData = $injector.get("$q").defer();
            deferredData.reject("simulates the call to rgisterImageCapture failed");
            sinon.stub(portalService, "registerImageCapture").returns(deferredData.promise);

            sinon.stub($uibModal, "open").returns({ close: function () { } });
            controller = $controller("CameraFeedsController", { $scope: scope, portalService: portalService, signalRService: signalRService, $window: $window, $uibModal: $uibModal });

            rootScope = $rootScope;
            $rootScope.$apply();
        });

        // Start recording
        controller.recording = true; // to validate the value has realy been changed
        controller.startRecording();
        scope.close(false); // force closing the parameters dialog
        rootScope.$digest(); // evaluate the promise used in startRecording()
        expect(controller.recording).to.equal(false);
    });
});