﻿"use strict";

describe("Cameras controller", function() {
    var controller, portalService, signalRService, $rootScope, $timeout, scope, sandbox, spySignalr, $translate, $httpBackend, setExposureSpy, $injector, mockFactory;
    var backendServerUrl = "http://localhost:8080";
    var backendLoggerUrl = "http://localhost:88";
    var dtoUserConfig = {
        data: {
            ExposureSettings: {
                ExposureMode: 'Automatic'
            },
            VehicleDetectionConfiguration: {
                ContextImageSize : {
                    Width: 1280,
                    Height: 960
                }
            }
        }
    };
    var dtoRegionOfInterest = { data: { Coordinates: [1, 2, 3, 4, 5, 6] } };
    var exposureCapabilities = {
        status: 200,
        data: {
            MinExposureCapability: 0,
            MaxExposureCapability: 40,
            MinGainCapability: 0.1,
            MaxGainCapability: 1
        }
    };
    var exposureSettings = {
        status: 200,
        data: {
            Gain: 10,
            GainMin: 2,
            GainMax: 23,
            ShutterTime: 0.2,
            ShutterTimeMin: 0.4,
            ShutterTimeMax: 0.9
        }
    };
    var dtoProperties = { data: false };

    beforeEach(function () {        
        module("portalApp.services", function ($provide) {
            $provide.value("backendServerUrl", backendServerUrl);
            $provide.value("backendLoggerUrl", backendLoggerUrl);
        });
        module("portalApp.controllers");
        module("pascalprecht.translate"); // for $translate
        module("portalApp.testing"); // load the module, otherwise: [$injector:unpr] Unknown provider: mockFactoryProvider <- mockFactory

        sandbox = sinon.sandbox.create();

        inject(function (_$injector_, _$translate_) {
            $injector = _$injector_;
            portalService = $injector.get("portalService");
            signalRService = $injector.get("signalRService");
            $rootScope = $injector.get("$rootScope");
            scope = $rootScope.$new();
            $timeout = $injector.get("$timeout");
            $translate = _$translate_;
            $httpBackend = $injector.get("$httpBackend");
            mockFactory = $injector.get("mockFactory");

            spySignalr = sandbox.stub(signalRService, "initializeFeedHub");

            var deferred = $injector.get("$q").defer();
            deferred.resolve(exposureCapabilities);
            sandbox.stub(portalService, "getExposureCapabilities").returns(deferred.promise);
            
            var deferredSettings = $injector.get("$q").defer();
            deferredSettings.resolve(exposureSettings);
            sandbox.stub(portalService, "getExposureDefaults").returns(deferredSettings.promise);

            var deferredSetExposure = $injector.get("$q").defer();
            deferredSetExposure.resolve(exposureSettings);
            setExposureSpy = sandbox.stub(portalService, "setExposureSettings").returns(deferredSetExposure.promise);
        });

        mockFactory.setSandbox(sandbox);
        mockFactory.stubGetUserConfig(dtoUserConfig);
        mockFactory.stubGetRegionOfInterest(dtoRegionOfInterest);
        mockFactory.stubGetProperties(dtoProperties);
    });

    afterEach(function () {
        sandbox.restore();
    });


    it("should be created successfully", function () {
        inject(function ($injector, $rootScope, $controller) {
            controller = $controller("CamerasController", { portalService: portalService, $scope: scope, $timeout: $timeout, signalRService: signalRService });
            $rootScope.$apply();
        });

        sinon.assert.calledOnce(spySignalr);
    });

    it("clearRegion should reset Coordinates array to empty and set form to dirty", function () {
        var spyDirty;
        inject(function ($injector, $rootScope, $controller) {
            controller = $controller("CamerasController", { portalService: portalService, $scope: scope, $timeout: $timeout, signalRService: signalRService });
            $rootScope.$apply();
        });
        
        controller.form = { $valid: true, $setDirty: function () { } };
        spyDirty = sandbox.stub(controller.form, "$setDirty");
        controller.clearRegion();
        expect(controller.userConfig.regionOfInterest.Coordinates).to.be.empty;
        sinon.assert.calledOnce(spyDirty);
    });

    it("cancel should set userConfig back to its original content and call form.$setPristine", function () {
        var spy, spyUpdate;
        inject(function ($injector, $rootScope, $controller) {
            controller = $controller("CamerasController", { portalService: portalService, $scope: scope, $timeout: $timeout, signalRService: signalRService });
            $rootScope.$apply();
            spyUpdate = sandbox.stub(controller, "updateExposureSettings");
        });

        controller.userConfig = { fake: "asdhasdhjashdjkhas" };
        controller.userConfig.regionOfInterest = { fake: [89, 52, 14] };
        controller.form = { $valid: true, $setPristine: function () { } };
        spy = sandbox.stub(controller.form, "$setPristine");
        controller.cancel();
        $rootScope.$apply();
        expect(angular.equals(controller.userConfig, dtoUserConfig));
        expect(angular.equals(controller.userConfig.regionOfInterest, dtoRegionOfInterest));
        sinon.assert.calledOnce(spy);
        sinon.assert.calledOnce(spyUpdate);
    });

    it("updateExposureSettings should call portalService.setExposureSettings to update the settings live", function () {
        inject(function ($injector, $rootScope, $controller) {
            controller = $controller("CamerasController", { portalService: portalService, $scope: scope, $timeout: $timeout, signalRService: signalRService });
            $rootScope.$apply();
        });
        var settings = { ExposureMode: "Manual", Gain: 99, ShutterTime: 0.5, CurrentIris: 10 };
        setExposureSpy.resetHistory();
        controller.updateExposureSettings(settings, null);
        $rootScope.$apply(); // need this to trigger promise execution   
        sinon.assert.calledOnce(setExposureSpy);
        assert(setExposureSpy.calledWith("Lpr Camera", settings));
    });

    it("updateExposureSettingsFromScope should call updateExposureSettings with shutter time max new value", function () {
        var spy;
        inject(function ($injector, $rootScope, $controller) {
            var dtoUserConfig = { data: { ExposureSettings: { Gain: 99 } } };
            mockFactory.stubGetUserConfig(dtoUserConfig);

            controller = $controller("CamerasController", { portalService: portalService, $scope: scope, $timeout: $timeout, signalRService: signalRService });
            $rootScope.$apply();

            spy = sandbox.stub(controller, "updateExposureSettings");
        });

        controller.currentExposureSettings = { ExposureMode: 'Custom', GainMax: 99, ShutterTimeMax: 88 };
        controller.updateExposureSettingsFromScope();
        sinon.assert.calledOnce(spy);
        expect(spy.args[0][0].ExposureMode).to.equal('Custom');
        expect(spy.args[0][0].GainMax).to.equal(99);
        expect(spy.args[0][0].ShutterTimeMax).to.equal(88);
    });

    it("updateExposureSettingsFromScope should call updateExposureSettings with shutter min new value", function () {
        var spy;
        inject(function ($injector, $rootScope, $controller) {
            var dtoUserConfig = { data: { ExposureSettings: { Gain: 99 } } };
            mockFactory.stubGetUserConfig(dtoUserConfig);

            controller = $controller("CamerasController", { portalService: portalService, $scope: scope, $timeout: $timeout, signalRService: signalRService });
            $rootScope.$apply();

            spy = sandbox.stub(controller, "updateExposureSettings");
        });

        controller.currentExposureSettings = { ExposureMode: 'Custom', GainMin: 1, ShutterTimeMin: 0.44 };
        controller.updateExposureSettingsFromScope();
        sinon.assert.calledOnce(spy);
        expect(spy.args[0][0].ExposureMode).to.equal('Custom');
        expect(spy.args[0][0].GainMin).to.equal(1);
        expect(spy.args[0][0].ShutterTimeMin).to.equal(0.44);
    });

    it("setupDrawing should react to mouse down event but ignore it with Coordinates array with more than 30 elements", function () {
        var document, spyDirty, event;
        inject(function ($injector, $rootScope, $controller) {
            document = $injector.get("$document");            
            document.find('body').append('<canvas id="regionCanvas" width="200" height="200"></canvas>');

            controller = $controller("CamerasController", { portalService: portalService, $scope: scope, $timeout: $timeout, signalRService: signalRService });
            $rootScope.$apply();
        });

        controller.userConfig.regionOfInterest.Coordinates = [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1];
        controller.form = { $valid: true, $setDirty: function () { } };
        spyDirty = sandbox.stub(controller.form, "$setDirty");
        var canvas = document.context.getElementById("regionCanvas");
        try {
            event = new MouseEvent('mousedown', {
                'view': window,
                'bubbles': false,
                'cancelable': true,
                'clientX': 48,
                'clientY': 102
            });
        } catch (e) {
            event = document.context.createEvent('MouseEvent');
            event.initMouseEvent('mousedown', false, true, window, 0, 0, 0, 48, 102, false, false, false, false, 0, null);
        }
        canvas.dispatchEvent(event);
        sinon.assert.notCalled(spyDirty);
        expect(controller.userConfig.regionOfInterest.Coordinates.length).to.equal(33);
        var dd = document.context.getElementById("regionCanvas");
        dd.parentNode.removeChild(dd);
    });

    it("saveCompleted should display an alert and set the form to pristine", function () {
        var spy;
        inject(function ($injector, $rootScope, $controller) {
            controller = $controller("CamerasController", { portalService: portalService, $scope: scope, $timeout: $timeout, signalRService: signalRService });
            $rootScope.$apply();
        });

        controller.form = { $valid: true, $setPristine: function () { } };
        spy = sandbox.stub(controller.form, "$setPristine");
        controller.saveCompleted();

        expect(controller.show).to.be.true;
        expect(controller.alert.type).to.equal('success');
        expect(controller.alert.msg).to.equal('STE_SENTENCE_ConfigurationSavedSuccessfully');
        $timeout.flush();
        expect(controller.show).to.be.false;
        sinon.assert.calledOnce(spy);
    });

    it("save should not call portalService.postRegionOfInterest but call portalService.setUserConfig when region of interest has not changed", function () {
        var spyRegion, spySaveCompleted;

        inject(function ($injector, $rootScope, $controller) {
            var defer = $injector.get("$q").defer();
            defer.resolve();
            spyRegion = sandbox.stub(portalService, "postRegionOfInterest").returns(defer.promise);
            
            controller = $controller("CamerasController", { portalService: portalService, $scope: scope, $timeout: $timeout, signalRService: signalRService });
            $rootScope.$apply();

            spySaveCompleted = sandbox.stub(controller, "saveCompleted");
        });

        var settings = {
            Gain: 10,
            GainMin: 2,
            GainMax: 23,
            ShutterTime: 0.2,
            ShutterTimeMin: 0.4,
            ShutterTimeMax: 0.9,
            ExposureMode: 'Automatic'
        }

        controller.currentExposureSettings = settings;
        controller.save();
        $rootScope.$digest();
        sinon.assert.called(setExposureSpy);
        sinon.assert.notCalled(spyRegion);
        sinon.assert.calledOnce(spySaveCompleted);
    });

    it("save should call portalService.postRegionOfInterest but not call portalService.setUserConfig when region of interest has changed", function () {
        var spyRegion, spySetConfig, spySaveCompleted;
        inject(function ($injector, $rootScope, $controller) {
            spyRegion = mockFactory.stubPostRegionOfInterest();
            spySetConfig = mockFactory.stubSetUserConfig();

            controller = $controller("CamerasController", { portalService: portalService, $scope: scope, $timeout: $timeout, signalRService: signalRService });
            $rootScope.$apply();

            spySaveCompleted = sandbox.stub(controller, "saveCompleted");
        });

        controller.userConfig.regionOfInterest = { Coordinates: [] };

        controller.save();
        $rootScope.$digest();
        sinon.assert.calledOnce(spyRegion);
        sinon.assert.notCalled(spySetConfig);
        sinon.assert.calledOnce(spySaveCompleted);
    });

});