﻿"use strict";

describe("Login controller", function () {
    var controller, portalService, focusFactory, $state, stateSpy, deferred, $rootScope, sandbox, $timeout;
    var backendServerUrl = "http://localhost:8080";
    var backendLoggerUrl = "http://localhost:88";

    beforeEach(function () {
        module("portalApp.services", function ($provide) {
            $provide.value("backendServerUrl", backendServerUrl);
            $provide.value("backendLoggerUrl", backendLoggerUrl);
        });
        module("portalApp.controllers");

        sandbox = sinon.sandbox.create();

        inject(function ($injector) {
            portalService = $injector.get("portalService");
            focusFactory = $injector.get("focusFactory");
            $state = $injector.get("$state");            
            deferred = $injector.get("$q").defer();
            $timeout = $injector.get("$timeout");
            $rootScope = $injector.get("$rootScope");
            stateSpy = sandbox.stub($state, "go");
            var d = $injector.get("$q").defer();
            d.resolve({ data: true });
            sinon.stub(portalService, "canReset").returns(d.promise);
        });
    });

    afterEach(function () {
        sandbox.restore();
    });
    
    it("should be created successfully", function () {
        inject(function($controller) {
            controller = $controller("LoginController", { $state: $state, portalService: portalService, focusFactory: focusFactory });
        });
        expect(controller).to.be.an('object');
        expect(controller.title).to.equal("LoginController");
        expect(controller.password).to.equal("");
    });

    it("login method should report password invalid when call fails", function () {
        inject(function ($controller) {
            controller = $controller("LoginController", { $state: $state, portalService: portalService, focusFactory: focusFactory });
        });
        deferred.reject();
        sinon.stub(portalService, "login").returns(deferred.promise);
        sinon.stub(focusFactory, "doFocus");
        var pwd = "patate";

        // setup password for login
        controller.password = pwd;
        // make the call.
        controller.login();

        $rootScope.$apply();

        expect(controller.error).to.be.true;
        expect(controller.password).to.equal("");

        assert(stateSpy.notCalled);
    });

    it("login method should not login if username is empty", function () {
        inject(function ($controller) {
            controller = $controller("LoginController", { $state: $state, portalService: portalService, focusFactory: focusFactory });
        });
        deferred.resolve({ data: true });
        var spyLogin = sinon.stub(portalService, "login").returns(deferred.promise);
        var pwd = "patate";
                        
        // setting an empty username will make the login abort
        controller.username = "";
        controller.password = pwd;
        // make the call.
        controller.login();

        $rootScope.$apply();

        expect(controller.username).to.equal("");
        expect(controller.password).to.equal("");
        expect(controller.error).to.be.true;

        assert(stateSpy.notCalled);
        assert(spyLogin.notCalled);
    });

    it("login method should change state to outofbox when password is Genetec and user is not prod", function () {
        inject(function ($controller) {
            controller = $controller("LoginController", { $state: $state, portalService: portalService, focusFactory: focusFactory });
        });
        deferred.resolve({ data: { accessToken: "blablabla", refreshToken: "blablabla", isOutOfBox: true } });
        sinon.stub(portalService, "login").returns(deferred.promise);
        var pwd = "Genetec";
        controller.username = "admin";
        controller.password = pwd;
        // make the call.
        controller.login();

        $rootScope.$apply();

        expect(controller.username).to.equal("admin");
        expect(controller.password).to.equal(pwd);
        expect(controller.error).to.be.false;

        sinon.assert.calledWith(stateSpy, "outofbox.welcome");
    });

    it("login method should change state to main.dashboard.overview when login succeeds and username is admin and reset the rootScope bodyClass", function () {
        sinon.stub(focusFactory, "doFocus");
        inject(function ($controller) {
            controller = $controller("LoginController", { $state: $state, portalService: portalService, focusFactory: focusFactory, $timeout: $timeout });
        });
        deferred.resolve({ data: { accessToken: "blablabla", refreshToken: "blablabla" } });
        sinon.stub(portalService, "login").returns(deferred.promise);
        
        var pwd = "patate";
        controller.username = "admin";
        controller.password = pwd;
        // make the call.
        controller.login();

        $rootScope.$digest();
        $timeout.flush();

        expect(controller.username).to.equal("admin");
        expect(controller.password).to.equal(pwd);
        expect(controller.error).to.be.false;

        sinon.assert.calledWith(stateSpy, "main.dashboard.overview");
        expect($rootScope.bodyClass).to.equal("");
    });

    it("login method should report error when login fails", function () {
        inject(function ($controller) {
            controller = $controller("LoginController", { $state: $state, portalService: portalService, focusFactory: focusFactory });
        });
        deferred.reject();
        sinon.stub(portalService, "login").returns(deferred.promise);
        var pwd = "patate";

        controller.password = pwd;
        // make the call.
        controller.login();

        $rootScope.$apply();

        expect(controller.username).to.equal("");
        expect(controller.password).to.equal("");
        expect(controller.error).to.be.true;

        assert(stateSpy.notCalled);
    });
});