﻿"use strict";

describe("Logs controller", function () {
    var controller, portalService, signalRService, rootScope, scope, sandbox, spySignalrLog, spySignalrActivities, spySignalrGetSources, spySignalrGetLogEntries, $translate, deferred, deferredSources, $timeout;
    var backendServerUrl = "http://localhost:8080";
    var backendLoggerUrl = "http://localhost:88";
    var userConfig = { data: { ReadModeConfiguration: { ReadingMode: 0 }, VehicleDetectionConfiguration: { ValidationRegion: { ValidationRegionType: 0, VertexList: [] }, ContextImagePointA: { X: 0, Y: 0 }, ContextImagePointB: { X: 0, Y: 0 }, LprImagePointA: { X: 0, Y: 0 }, LprImagePointB: { X: 0, Y: 0 }, ContextImageSize: {} } } };

    beforeEach(function () {
        module("portalApp.services", function ($provide) {
            $provide.value("backendServerUrl", backendServerUrl);
            $provide.value("backendLoggerUrl", backendLoggerUrl);
            $provide.value("maxLogMessages", 2);
        });
        module("portalApp.controllers");
        module("portalApp.filters");
        module("pascalprecht.translate"); // for $translate

        sandbox = sinon.sandbox.create();

        $translate = function (translation) {
            return {
                then: function (callback) {
                    return callback("Translate: " + translation);
                }
            }
        };

        $translate.isReady = function () { return false; }

        inject(function ($injector, $rootScope, _$translate_) {
            portalService = $injector.get("portalService");
            signalRService = $injector.get("signalRService");
            $timeout = $injector.get("$timeout");
            scope = $rootScope.$new();
            deferred = $injector.get("$q").defer();
            deferredSources = $injector.get("$q").defer();

            deferred.resolve({ data: true });
            deferredSources.resolve([]);

            spySignalrLog = sandbox.stub(signalRService, "initializeLogHub").returns(deferred.promise);
            spySignalrActivities = sandbox.stub(signalRService, "initializeLastActivitiesHub");
            spySignalrGetSources = sandbox.stub(signalRService, "getSources").returns(deferredSources.promise);
            spySignalrGetLogEntries = sandbox.stub(signalRService, "getLogEntries").returns(deferredSources.promise);


            // Mock the getUserConfig service
            var d = $injector.get("$q").defer();
            d.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(d.promise);
        });
    });

    afterEach(function () {
        sandbox.restore();
    });
    
    it("should be created successfully", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("LogsController", { portalService: portalService, signalRService: signalRService, $scope: scope });
            $rootScope.$apply();
        });
        expect(controller).to.be.an('object');
        expect(controller.title).to.equal("LogsController");
        sinon.assert.calledOnce(spySignalrLog);
        sinon.assert.calledOnce(spySignalrActivities);
    });

    it("should update timesearchOptions when LastActivities is emitted", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("LogsController", { portalService: portalService, signalRService: signalRService, $scope: scope, $translate: $translate });
            rootScope = $rootScope;
            $rootScope.$apply();
        });

        var message = {
            LastReboot: "Value1",
            LastAppRestart: "Value2"
        }

        expect(controller.timeSearchOptions.length).to.equal(6);
        rootScope.$emit("LastActivities", message);
        expect(controller.timeSearchOptions.length).to.equal(8);
        expect(controller.timeSearchOptions[6].value).to.equal("moment(\"" + message.LastReboot + "\")");
        expect(controller.timeSearchOptions[7].value).to.equal("moment(\"" + message.LastAppRestart + "\")");
    });

    it("should update timesearchOptions when LastActivities is emitted twice by updating the values", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("LogsController", { portalService: portalService, signalRService: signalRService, $scope: scope, $translate: $translate });
            rootScope = $rootScope;
            $rootScope.$apply();
        });

        var message = {
            LastReboot: "Value1",
            LastAppRestart: "Value2"
        }

        var message2 = {
            LastReboot: "Value3",
            LastAppRestart: "Value4"
        }

        rootScope.$emit("LastActivities", message);
        expect(controller.timeSearchOptions.length).to.equal(8);
        expect(controller.timeSearchOptions[6].value).to.equal("moment(\"" + message.LastReboot + "\")");
        expect(controller.timeSearchOptions[7].value).to.equal("moment(\"" + message.LastAppRestart + "\")");
        rootScope.$emit("LastActivities", message2);
        expect(controller.timeSearchOptions.length).to.equal(8);
        expect(controller.timeSearchOptions[6].value).to.equal("moment(\"" + message2.LastReboot + "\")");
        expect(controller.timeSearchOptions[7].value).to.equal("moment(\"" + message2.LastAppRestart + "\")");
    });

    it("should add an entry when addEntry is emitted", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("LogsController", { portalService: portalService, signalRService: signalRService, $scope: scope });
            rootScope = $rootScope;
            $rootScope.$apply();
        });

        var message = { Source: "Something", Severity: "Error" };
        scope.logs = [];
        controller.searchCriteria.Source = "Something";        

        expect(controller.logEntries.length).to.equal(0);
        rootScope.$emit("addEntry", message);
        expect(controller.logEntries.length).to.equal(1);
        expect(controller.logEntries[0]).to.equal(message);
    });

    it("should not add an entry when addEntry is emitted if maxLogMessages is reached", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("LogsController", { portalService: portalService, signalRService: signalRService, $scope: scope });
            rootScope = $rootScope;
            $rootScope.$apply();
        });

        var message = { Source: "Something", Severity: "Error" };
        scope.logs = [];
        controller.searchCriteria.Source = "Something";

        expect(controller.logEntries.length).to.equal(0);
        rootScope.$emit("addEntry", message);
        rootScope.$emit("addEntry", message);
        expect(controller.logEntries.length).to.equal(2);
        rootScope.$emit("addEntry", message);
        expect(controller.logEntries.length).to.equal(2);
    });

    it("toggle method should pause if unpaused", function () {
        var deffered;        
        inject(function ($injector, $controller, $rootScope) {
            deffered = $injector.get("$q").defer();
            deffered.resolve({});
            controller = $controller("LogsController", { portalService: portalService, signalRService: signalRService, $scope: scope });
            $rootScope.$apply();            
        });

        var spyPause = sandbox.stub(signalRService, "pauseLogs").returns(deffered.promise);
        var spyResume = sandbox.stub(signalRService, "resumeLogs").returns(deffered.promise);

        controller.isPaused = false;
        controller.toggle();
        expect(controller.isPaused).to.equal(true);
        sinon.assert.calledOnce(spyPause);
        sinon.assert.notCalled(spyResume);
    });

    it("toggle method should resume if paused", function () {
        var deffered;

        inject(function ($injector, $controller, $rootScope) {
            deffered = $injector.get("$q").defer();
            deffered.resolve({});
            controller = $controller("LogsController", { portalService: portalService, signalRService: signalRService, $scope: scope });
            $rootScope.$apply();
        });

        var spyPause = sandbox.stub(signalRService, "pauseLogs").returns(deffered.promise);
        var spyResume = sandbox.stub(signalRService, "resumeLogs").returns(deffered.promise);

        controller.isPaused = true;
        controller.toggle();
        expect(controller.isPaused).to.equal(false);
        sinon.assert.notCalled(spyPause);
        sinon.assert.calledOnce(spyResume);
    });

    it("updateSourceRegistration method should enable source", function () {
        var deffered;

        inject(function ($injector, $controller, $rootScope) {
            deffered = $injector.get("$q").defer();
            deffered.resolve({});
            controller = $controller("LogsController", { portalService: portalService, signalRService: signalRService, $scope: scope });
            $rootScope.$apply();
        });

        var source = {
            IsActive : true,
            Name: "Source1"
        }
        var spyEnable = sandbox.stub(signalRService, "enableSource").returns(deffered.promise);
        var spyDisable = sandbox.stub(signalRService, "disableSource").returns(deffered.promise);
        controller.updateSourceRegistration(source);
        sinon.assert.notCalled(spyDisable);
        sinon.assert.calledOnce(spyEnable);
    });

    it("updateSourceRegistration method should disable source", function () {
        var deffered;

        inject(function ($injector, $controller, $rootScope) {
            deffered = $injector.get("$q").defer();
            deffered.resolve({});
            controller = $controller("LogsController", { portalService: portalService, signalRService: signalRService, $scope: scope });
            $rootScope.$apply();
        });

        var source = {
            IsActive: false,
            Name: "Source1"
        }
        var spyEnable = sandbox.stub(signalRService, "enableSource").returns(deffered.promise);
        var spyDisable = sandbox.stub(signalRService, "disableSource").returns(deffered.promise);
        controller.updateSourceRegistration(source);
        sinon.assert.calledOnce(spyDisable);
        sinon.assert.notCalled(spyEnable);
    });

    it("updatedSearchOption method should update timeSearchOption", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("LogsController", { portalService: portalService, signalRService: signalRService, $scope: scope });
            $rootScope.$apply();
        });

        controller.timeSearchOption = undefined;
        controller.updatedSearchOption("Test1234");
        expect(controller.timeSearchOption).to.equal("Test1234");
    });

    it("search method should update logEntries", function () {
        var d;
        inject(function ($injector, $controller, $rootScope) {
            d = $injector.get("$q").defer();
            controller = $controller("LogsController", { portalService: portalService, signalRService: signalRService, $scope: scope });
            rootScope = $rootScope;
            $rootScope.$apply();
        });

        var entry = { Source: "Something", Severity: "Error", Message: "Allo toi" };
        spySignalrGetLogEntries.restore();
        d.resolve([entry]);
        spySignalrGetLogEntries = sandbox.stub(signalRService, "getLogEntries").returns(d.promise);

        expect(controller.logEntries.length).to.equal(0);
        controller.search();
        rootScope.$apply(); // need this to trigger promise execution
        $timeout.flush(); // need this to trigger timeout exectution

        expect(controller.logEntries.length).to.equal(1);
        expect(controller.logEntries[0]).to.equal(entry);
    });

    it("search method should not update logEntries when getLogEntries fails", function () {
        var d;
        inject(function ($injector, $controller, $rootScope) {
            d = $injector.get("$q").defer();
            controller = $controller("LogsController", { portalService: portalService, signalRService: signalRService, $scope: scope });
            rootScope = $rootScope;
            $rootScope.$apply();
        });

        spySignalrGetLogEntries.restore();
        d.reject();
        spySignalrGetLogEntries = sandbox.stub(signalRService, "getLogEntries").returns(d.promise);

        expect(controller.logEntries.length).to.equal(0);
        controller.search();
        rootScope.$apply(); // need this to trigger promise execution
        $timeout.flush(); // need this to trigger timeout exectution

        expect(controller.logEntries.length).to.equal(0);
    });

    it("loadMoreLogs method should add log entry", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("LogsController", { portalService: portalService, signalRService: signalRService, $scope: scope });
            $rootScope.$apply();
        });

        controller.logEntries = [{ Source: "Something", Severity: "Error", Message: "Allo toi" }];
        scope.logs = [];
        controller.loadMoreLogs();
        expect(scope.logs.length).to.equal(1);
    });

    it("should set translated timeSearchOptions when $translateChangeSuccess is emitted", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("LogsController", { portalService: portalService, signalRService: signalRService, $scope: scope });
            rootScope = $rootScope;
            $rootScope.$apply();
        });

        expect(controller.timeSearchOptions.length).to.equal(6);
        expect(controller.timeSearchOptions[0].displayMember).to.equal("Last hour");
        expect(controller.timeSearchOptions[1].displayMember).to.equal("Last 2 hours");
        expect(controller.timeSearchOptions[2].displayMember).to.equal("Last 24 hours");
        expect(controller.timeSearchOptions[3].displayMember).to.equal("Last 48 hours");
        expect(controller.timeSearchOptions[4].displayMember).to.equal("Last week");
        expect(controller.timeSearchOptions[5].displayMember).to.equal("All");
        rootScope.$emit("$translateChangeSuccess", {});
        expect(controller.timeSearchOptions[0].displayMember).to.equal("STE_LABEL_LastHour");
        expect(controller.timeSearchOptions[1].displayMember).to.equal("STE_LABEL_Last2Hours");
        expect(controller.timeSearchOptions[2].displayMember).to.equal("STE_LABEL_Last24Hours");
        expect(controller.timeSearchOptions[3].displayMember).to.equal("STE_LABEL_Last48Hours");
        expect(controller.timeSearchOptions[4].displayMember).to.equal("STE_LABEL_LastWeek");
        expect(controller.timeSearchOptions[5].displayMember).to.equal("STE_LABEL_AnyTime");
    });

    it("should add sources when getSources succeeds", function () {
        var d;
        
        inject(function ($injector, $controller, $rootScope) {
            spySignalrGetSources.restore();
            d = $injector.get("$q").defer();
            d.resolve([{ IsActive: true, ExpiresOn: "2000-01-01" }]);

            spySignalrGetSources = sandbox.stub(signalRService, "getSources").returns(d.promise);

            controller = $controller("LogsController", { portalService: portalService, signalRService: signalRService, $scope: scope });
            $rootScope.$apply();
        });
        
        expect(controller.sources.length).to.equal(1);        
    });

    it("should call showStickyAlert when LoggerServiceConnectionFailure is emitted and rootScope.connectionLost is set to true", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("LogsController", { portalService: portalService, signalRService: signalRService, $scope: scope });
            rootScope = $rootScope;
            $rootScope.$apply();
        });

        controller.isAlertShown = false;
        rootScope.$emit("LoggerServiceConnectionFailure", {});
        expect(rootScope.connectionLost).to.be.true;
    });

    it("should call setupSources when LoggerServiceConnectionSuccess is emitted", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("LogsController", { portalService: portalService, signalRService: signalRService, $scope: scope });
            rootScope = $rootScope;
            $rootScope.$apply();
        });

        controller.isAlertShown = false;
        var spy = sandbox.stub(controller, "setupSources");
        rootScope.$emit("LoggerServiceConnectionSuccess", {});
        expect(rootScope.connectionLost).to.be.false;
        sinon.assert.calledOnce(spy);
    });
});