﻿"use strict";

describe("Main controller", function () {
    var controller, $state, scope, sandbox, $translate, localizationFactory, portalService;
    var backendServerUrl = "http://localhost:8080";
    var backendLoggerUrl = "http://localhost:88";
    var mockNavigator = {
        language: "en-US"
        // don't care about userLanguage
    };
    var mockUserConfig = {
        data: {
            ReadModeConfiguration: {
                ReadingMode: '' //!!!
            },
            NetworkConfiguration: {
                NetworkType: 0 // DHCP
            }
        }
    };

    var mockMoment = {
        locale: function (langKey) { }
    };

    beforeEach(function () {
        module("portalApp.services", function ($provide) {
            $provide.value("backendServerUrl", backendServerUrl);
            $provide.value("backendLoggerUrl", backendLoggerUrl);
        });
        module("portalApp.controllers");
        module("portalApp.factories", function ($provide) {
            $provide.value("navigator", mockNavigator);
            $provide.value("moment", mockMoment);
        });
        module("pascalprecht.translate"); // for $translate

        sandbox = sinon.sandbox.create();

        inject(function ($injector, $rootScope, _$translate_) {
            localizationFactory = $injector.get("localizationFactory");
            portalService = $injector.get("portalService");
            $state = $injector.get("$state");
            scope = $rootScope.$new();
            $translate = _$translate_;

            // Set UserConfig
            var deferred = $injector.get("$q").defer();
            deferred.resolve(mockUserConfig);
            sandbox.stub(portalService, "getUserConfig").returns(deferred.promise);
        });
    });

    afterEach(function () {
        sandbox.restore();
    });
    
    it("should be created successfully", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("MainController", { localizationFactory: localizationFactory, $scope: scope, portalService: portalService });
            $rootScope.$apply();
        });
        expect(controller).to.be.an('object');
        expect(controller.title).to.equal("MainController");

    });

    it("setActiveMainMenu method should not set any class as active or collapse in when passed a path that is not the current state", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("MainController", { localizationFactory: localizationFactory, $scope: scope, portalService: portalService });
            $rootScope.$apply();
        });

        $state.current.name = "Main";
        var fakeState = { name: 'Patate' };
        controller.setActiveMainMenu(fakeState);
        angular.forEach(controller.topMenuClasses, function (value, key) {
            expect(value).to.equal("");
        });
        angular.forEach(controller.sideMenuClasses, function (value, key) {
            expect(value).to.equal("");
        });
        angular.forEach(controller.collapsableSideMenuClasses, function (value, key) {
            expect(value).to.equal("collapse");
        });
    });

    it("setActiveMainMenu method should set top menu class and but no side menu as active string when passed a path that is the current state", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("MainController", { localizationFactory: localizationFactory, $scope: scope, portalService: portalService });
            $rootScope.$apply();
        });

        $state.current.name = "main.dashboard";
        var fakeState = { name: 'main.dashboard.dummy' };
        controller.setActiveMainMenu(fakeState);
        angular.forEach(controller.topMenuClasses, function (value, key) {
            if (fakeState.name.indexOf(key) > -1)
                expect(controller.topMenuClasses[key]).to.equal("active");
            else
                expect(controller.topMenuClasses[key]).to.equal("");
        });
        angular.forEach(controller.sideMenuClasses, function (value, key) {
            expect(value).to.equal("");
        });
        angular.forEach(controller.collapsableSideMenuClasses, function (value, key) {
            expect(value).to.equal("collapse");
        });
    });

    it("setActiveMainMenu method should set top menu class and side menu as active string when passed a path that is the current state", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("MainController", { localizationFactory: localizationFactory, $scope: scope, portalService: portalService });
            $rootScope.$apply();
        });

        $state.current.name = "main.dashboard";
        var fakeState = { name: 'main.dashboard.overview' };
        controller.setActiveMainMenu(fakeState);
        angular.forEach(controller.topMenuClasses, function (value, key) {
            if (fakeState.name.indexOf(key) > -1)
                expect(controller.topMenuClasses[key]).to.equal("active");
            else
                expect(controller.topMenuClasses[key]).to.equal("");
        });
        angular.forEach(controller.sideMenuClasses, function (value, key) {
            if (fakeState.name === key)
                expect(value).to.equal("active");
            else
                expect(value).to.equal("");
        });
        angular.forEach(controller.collapsableSideMenuClasses, function (value, key) {
            expect(value).to.equal("collapse");
        });
    });

    it("getCurrentLanguage method should return language key from localizationFactory", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("MainController", { localizationFactory: localizationFactory, $scope: scope, portalService: portalService });
            $rootScope.$apply();
            $rootScope.connectionLost = false;
        });

        var fakeData = { langKey : 'toto'};

        sandbox.stub(localizationFactory, "getData").returns(fakeData);
        
        var value = controller.currentLanguage();
        expect(value).to.equal("toto");
    });

    it("help method should try to open the admin guide in another window but fails and open an alert window", function () {
        var spyOpen, spyAlert;
        inject(function ($injector, $controller, $rootScope) {
            var $window = $injector.get("$window");
            spyOpen = sandbox.stub($window, "open").returns(null);
            spyAlert = sandbox.stub($window, "alert");
            controller = $controller("MainController", { localizationFactory: localizationFactory, $scope: scope, $window: $window, portalService: portalService });
            $rootScope.$apply();
            $rootScope.connectionLost = false;
        });

        controller.help();
        sinon.assert.calledOnce(spyOpen);
        sinon.assert.calledOnce(spyAlert);
    });

    it("help method should open the admin guide in another window and not open an alert window", function () {
        var spyOpen, spyAlert;
        inject(function ($injector, $controller, $rootScope) {
            var $window = $injector.get("$window");
            spyOpen = sandbox.stub($window, "open").returns({ focus: function(){} });
            spyAlert = sandbox.stub($window, "alert");
            controller = $controller("MainController", { localizationFactory: localizationFactory, $scope: scope, $window: $window, portalService: portalService });
            $rootScope.$apply();
            $rootScope.connectionLost = false;
        });

        controller.help();
        sinon.assert.calledOnce(spyOpen);
        sinon.assert.notCalled(spyAlert);
    });
});