﻿"use strict";

describe("OutOfBox controller", function () {
    var controller, $state, scope, sandbox, $translate, portalService, signalRService, $interval;
    var backendServerUrl = "http://localhost:8080";
    var backendLoggerUrl = "http://localhost:88";
    var userConfig = { data: 51353541541 };

    beforeEach(function () {
        module("portalApp.services", function ($provide) {
            $provide.value("backendServerUrl", backendServerUrl);
            $provide.value("backendLoggerUrl", backendLoggerUrl);
        });
        module("portalApp.controllers");
        module("pascalprecht.translate"); // for $translate

        sandbox = sinon.sandbox.create();

        inject(function ($injector, $rootScope, _$translate_, _$interval_) {
            portalService = $injector.get("portalService");
            signalRService = $injector.get("signalRService");           
            $state = $injector.get("$state");
            scope = $rootScope.$new();
            $translate = _$translate_;
            $interval = _$interval_;
        });
    });

    afterEach(function () {
        sandbox.restore();
    });
    
    it("should be created successfully", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("OutOfBoxController", { portalService: portalService, signalRService: signalRService, $scope: scope, userConfig: userConfig });
            $rootScope.$apply();
        });
        expect(controller).to.be.an('object');
        expect(controller.title).to.equal("OutOfBoxController");
    });

    it("showAlert method with timeout argument should stop displaying the message after timeout", function () {
        var timeout;
        inject(function ($injector, $controller, $rootScope) {
            timeout = $injector.get("$timeout");
            controller = $controller("OutOfBoxController", { portalService: portalService, signalRService: signalRService, $scope: scope, userConfig: userConfig });
            $rootScope.$apply();
        });

        controller.show = true;
        controller.showAlert("test", "allo", true);
        expect(controller.showMessage).to.be.true;
        expect(controller.message.type).to.equal("test");
        expect(controller.message.text).to.equal("allo");
        timeout.flush();
        expect(controller.showMessage).to.be.false;
    });

    it("save method should set user config then change state to outofbox.completed", function () {
        var window = { sessionStorage: { accessToken: "Bonjour", refreshToken: "Allo" } };
        var rootScope;
        inject(function ($injector, $controller, $rootScope) {
            var deferred = $injector.get("$q").defer();
            deferred.resolve({ data: { accessToken: "Test", refreshToken: "Patate" } });
            sandbox.stub(portalService, "setUserConfig").returns(deferred.promise);
            sandbox.stub(portalService, "setPassword").returns(deferred.promise);
            controller = $controller("OutOfBoxController", { portalService: portalService, signalRService: signalRService, $scope: scope, userConfig: userConfig, $window: window });
            $rootScope.$apply();
            rootScope = $rootScope;
        });

        var stateSpy = sandbox.stub($state, "go");
        controller.newPassword2 = "Genetec1! 2dsfjhdfjh334873849@#$#(";
        controller.save();
        rootScope.$apply(); // need this to trigger promise execution     
        sinon.assert.calledWith(stateSpy, "outofbox.completed");
    });    

    it("save method should fail to set user config and not update accesstoken and refreshtoken", function () {
        var window = { sessionStorage: { accessToken: "Bonjour", refreshToken: "Allo" } };
        var rootScope;
        inject(function ($injector, $controller, $rootScope) {
            var deferred = $injector.get("$q").defer();
            deferred.reject();
            sandbox.stub(portalService, "setUserConfig").returns(deferred.promise);
            sandbox.stub(portalService, "setPassword").returns(deferred.promise);
            var d = $injector.get("$q").defer();
            d.resolve({ data: "False"});
            sandbox.stub(portalService, "getPoePlusStatus").returns(d.promise);
            controller = $controller("OutOfBoxController", { portalService: portalService, signalRService: signalRService, $scope: scope, userConfig: userConfig, $window: window });
            $rootScope.$apply();
            rootScope = $rootScope;
        });

        controller.newPassword2 = "Genetec1! 2dsfjhdfjh334873849@#$#(";
        controller.save();
        rootScope.$apply(); // need this to trigger promise execution
        expect(window.sessionStorage.accessToken).to.equal("Bonjour");
        expect(window.sessionStorage.refreshToken).to.equal("Allo");
    });

    it("nextState method when current state is outofbox.welcome should change state to outofbox.power", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("OutOfBoxController", { portalService: portalService, signalRService: signalRService, $scope: scope, userConfig: userConfig });
            $rootScope.$apply();
        });
        
        var stateSpy = sandbox.stub($state, "go");
        $state.current.name = "outofbox.welcome";
        controller.nextState();
        sinon.assert.calledWith(stateSpy, "outofbox.power");
    });    

    it("nextState method when current state is outofbox.power should change state to outofbox.password", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("OutOfBoxController", { portalService: portalService, signalRService: signalRService, $scope: scope, userConfig: userConfig });
            $rootScope.$apply();
        });

        var stateSpy = sandbox.stub($state, "go");
        $state.current.name = "outofbox.power";
        controller.nextState();
        sinon.assert.calledWith(stateSpy, "outofbox.password");
    });

    it("nextState method when current state is outofbox.completed should delete both tokens change state to login", function () {
        var window = { sessionStorage: { accessToken: "Bonjour", refreshToken: "Allo" }, location: { href: "Mourial", reload: function (value) { } } };
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("OutOfBoxController", { portalService: portalService, signalRService: signalRService, $scope: scope, userConfig: userConfig, $window: window });
            $rootScope.$apply();
        });

        var stateSpy = sandbox.stub($state, "go");
        $state.current.name = "outofbox.completed";
        controller.nextState();
        expect(window.sessionStorage.accessToken).to.be.undefined;
        expect(window.sessionStorage.refreshToken).to.be.undefined;
        sinon.assert.calledWith(stateSpy, "login");
    });        

    it("previousState method when current state is outofbox.password should change state to outofbox.power and passwords cleared", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("OutOfBoxController", { portalService: portalService, signalRService: signalRService, $scope: scope, userConfig: userConfig});
            $rootScope.$apply();
        });

        var stateSpy = sandbox.stub($state, "go");
        controller.newPassword1 = "wrong"; // will be cleared
        controller.newPassword2 = "wrong"; // will be cleared
        $state.current.name = "outofbox.password";
        controller.previousState();
        sinon.assert.calledWith(stateSpy, "outofbox.power");
        expect(controller.newPassword1).to.equal("");
        expect(controller.newPassword2).to.equal("");
    });    

    it("previousState method when current state is outofbox.power should change state to outofbox.welcome", function () {
        inject(function ($injector, $controller, $rootScope) {
            controller = $controller("OutOfBoxController", { portalService: portalService, signalRService: signalRService, $scope: scope, userConfig: userConfig });
            $rootScope.$apply();
        });

        var stateSpy = sandbox.stub($state, "go");
        $state.current.name = "outofbox.power";
        controller.previousState();
        sinon.assert.calledWith(stateSpy, "outofbox.welcome");
    });
});