﻿"use strict";

describe("Portal Directive Module", function () {

    var $compile, $rootScope, scope, $translate;

    beforeEach(function () {
        module("portalApp.directives");
        module("pascalprecht.translate");

        inject(function (_$compile_, _$rootScope_, _$translate_) {
            $compile = _$compile_;
            $rootScope = _$rootScope_;
            $translate = _$translate_;
        });
    });

    it("should define and use 'ipv4AndSubnetMaskValidator' directive to only allow a valid IP address " +
        "(address finishing with 0 or 255 are allowed, but network and broadcast address aren't)", function () {

            var scope = $rootScope.$new();

            scope.vm = { ip: null, mask: null };

            var element =
                '<form name="f">' +
                '   <input type="text" name="ip" ng-model="vm.ip" ipv4-and-subnet-mask-validator subnet-mask="{{vm.mask}}" is-subnet-mask="true">' +
                '</form>';

            $compile(element)(scope);
            var form = scope.f;

            // IP address valid
            scope.vm.mask = "255.255.255.0";
            form.ip.$setViewValue("192.168.10.25");
            form.ip.$validate();
            expect(form.$valid).to.be.true;

            // IP address invalid (equals broadcast address)
            scope.vm.mask = "255.255.255.0";
            form.ip.$setViewValue("192.168.10.255");
            form.ip.$validate();
            expect(form.$valid).to.be.false;

            // IP address invalid (equals network address)
            scope.vm.mask = "255.255.255.0";
            form.ip.$setViewValue("192.168.10.0");
            form.ip.$validate();
            expect(form.$valid).to.be.false;

            // IP address valid (ending with 0)
            scope.vm.mask = "255.255.0.0";
            form.ip.$setViewValue("192.168.10.0");
            form.ip.$validate();
            expect(form.$valid).to.be.true;

            // IP address valid (ending with 255)
            scope.vm.mask = "255.255.0.0";
            form.ip.$setViewValue("192.168.10.255");
            form.ip.$validate();
            expect(form.$valid).to.be.true;

            // IP address invalid (equals broadcast address)
            scope.vm.mask = "0.0.0.0";
            form.ip.$setViewValue("255.255.255.255");
            form.ip.$validate();
            expect(form.$valid).to.be.false;

            // IP address invalid (single host mask)
            scope.vm.mask = "255.255.255.255";
            form.ip.$setViewValue("0.0.0.0");
            form.ip.$validate();
            expect(form.$valid).to.be.false;

            element =
                '<form name="f">' +
                '   <input type="text" name="mask" ng-model="vm.mask" ipv4-and-subnet-mask-validator ip-address="{{vm.ip}}" is-subnet-mask="false">' +
                '</form>';

            $compile(element)(scope);
            form = scope.f;

            // IP address invalid (testing the update of the subnet mask field instead of the ip)
            scope.vm.ip = "10.10.3.255";
            form.mask.$setViewValue("255.255.252.0");
            form.mask.$validate();
            expect(form.$valid).to.be.false;

            // IP address valid (testing the update of the subnet mask field instead of the ip)
            scope.vm.ip = "10.10.0.255";
            form.mask.$setViewValue("255.255.252.0");
            form.mask.$validate();
            expect(form.$valid).to.be.true;
        });


    it("should define and use 'gnEqualsTo' directive to accept identical values", function () {

        var scope = $rootScope.$new();

        scope.vm = { newPassword1: "abc", newPassword2: null };

        var element =
            '<form name="f">' +
            '   <input type="text" name="newPwd2" ng-model="vm.newPassword2" gn-equals-to="vm.newPassword1"></input>' +
            '</form>'

        $compile(element)(scope);
        var form = scope.f;

        // Set identical values
        var newValue = "abc";
        form.newPwd2.$setViewValue(newValue);
        scope.$digest(); // make value 'dirty' (not 'prestine')
        expect(scope.vm.newPassword2).to.be.equal(newValue);
        expect(form.newPwd2.$valid).to.be.true;
    });

    it("should define and use 'gnEqualsTo' directive to reject different values", function () {

        var scope = $rootScope.$new();

        scope.vm = { newPassword1: "abc", newPassword2: null };

        var element =
            '<form name="f">' +
            '   <input type="text" name="newPwd2" ng-model="vm.newPassword2" gn-equals-to="vm.newPassword1"/>' +
            '</form>'

        $compile(element)(scope);
        var form = scope.f;

        // Set different values
        var newValue = "xyz";
        form.newPwd2.$setViewValue(newValue);
        scope.$digest(); // make value 'dirty' (not 'prestine')
        expect(scope.vm.newPassword2).to.be.undefined; // value is not set when in error
        expect(form.newPwd2.$valid).to.be.false;
    });


    it("should define and use 'gnDifferentFrom' directive to reject identical values", function () {

        var scope = $rootScope.$new();

        scope.vm = { oldPassword: "abc", newPassword: null };

        var element =
            '<form name="f">' +
            '   <input type="text" name="newPwd" ng-model="vm.newPassword" gn-different-from="vm.oldPassword"/>' +
            '</form>'

        $compile(element)(scope);
        var form = scope.f;

        // Set identical values
        var newValue = "abc";
        form.newPwd.$setViewValue(newValue);
        scope.$digest(); // make value 'dirty' (not 'prestine')

        expect(scope.vm.newPassword).to.be.undefined; // value is not set when in error
        expect(form.newPwd.$valid).to.be.false;
    });

    it("should define and use 'gnDifferentFrom' directive to accept different values", function () {

        var scope = $rootScope.$new();

        scope.vm = { oldPassword: "abc", newPassword: null };

        var element =
            '<form name="f">' +
            '   <input type="text" name="newPwd" ng-model="vm.newPassword" gn-different-from="vm.oldPassword"/>' +
            '</form>'

        $compile(element)(scope);
        var form = scope.f;

        // Set different values
        var newValue = "xyz";
        form.newPwd.$setViewValue(newValue);
        scope.$digest(); // make value 'dirty' (not 'prestine')

        expect(scope.vm.newPassword).to.be.equal(newValue);
        expect(form.newPwd.$valid).to.be.true;
    });


    //it("should define and use 'gnCustomOnChange' directive", function () {

    //    // This test fails. Probably because it needs the DOM infrastructure to dispatch events
    //    var called = false;

    //    scope = $rootScope.$new();

    //    scope.vm = { onChange: function () { called = true; }, location: null };

    //    var element =
    //        '<form name="f">' +
    //        '   <input type="text" name="xxx" ng-model="vm.location" gn-custom-on-change="vm.onChange"/>' +
    //        '</form>'

    //    $compile(element)(scope);
    //    var form = scope.f;

    //    scope.$apply();
    //    expect(called).to.be.false;

    //    var newValue = "changed";
    //    form.xxx.$setViewValue(newValue); // changing the <input> value should trigger onChange (and set called to true)
    //    scope.$apply();

    //    expect(scope.vm.location).to.be.equal(newValue);
    //    //!!!onChange is not called: expect(called).to.be.true;
    //});

    it("should define and use 'gnWithFloatingLabel' directive", function () {

        var scope = $rootScope.$new();

        scope.vm = { server: "" };

        var element =
            '<input type="text" id="xxx" placeHolder="Server" ng-model="vm.server" gn-with-floating-label="f"/>';

        var content = $compile(element)(scope);

        scope.$digest();

        var inputElem = content[0];
        var divElem = inputElem.nextSibling;

        expect(divElem.nextSibling).to.be.null;
        expect(inputElem.tagName).to.equal("INPUT");
        expect(divElem.tagName).to.equal("DIV");

        expect(divElem.outerHTML).to.equal('<div class="floating-label ng-scope" ng-class="{ \'has-error\' : f.xxx.$invalid &amp;&amp; f.xxx.$dirty }" translate="Server">Server</div>');
    });

    // That test does not run properly.
    //it("should define and use 'gnFloatingLabelWithIcon' directive (without icon => same as gnWithFloatingLabel)", function () {

    //    var scope = $rootScope.$new();

    //    scope.vm = { server: "" };

    //    var element =
    //        '<input type="text" id="xxx" placeHolder="Server" ng-model="vm.server" gn-floating-label-with-icon="f"/>';

    //    var content = $compile(element)(scope);

    //    scope.$digest();

    //    var inputElem = content[0];
    //    var divElem = inputElem.nextSibling;

    //    expect(divElem.nextSibling).to.be.null;
    //    expect(inputElem.tagName).to.equal("INPUT");
    //    expect(divElem.tagName).to.equal("DIV");

    //    expect(divElem.outerHTML).to.equal('<div class="floating-label ng-scope" ng-class="{ \'has-error\' : f.xxx.$invalid &amp;&amp; f.xxx.$dirty }" translate="Server">Server</div>');
    //});

    // That test does not run properly.
    //it("should define and use 'gnFloatingLabelWithIcon' directive (with icon)", function () {

    //    var scope = $rootScope.$new();

    //    scope.vm = { server: "" };

    //    var element =
    //        '<input type="text" id="xxx" placeHolder="Server" ng-model="vm.server"' +
    //            'icon="<i class=\'fa fa-wifi\'></i>"/> gn-floating-label-with-icon="f"';

    //    var content = $compile(element)(scope);

    //    scope.$digest();

    //    var inputElem = content[0];
    //    var divElem = inputElem.nextSibling;

    //    expect(divElem.nextSibling).to.be.null;
    //    expect(inputElem.tagName).to.equal("INPUT");
    //    expect(divElem.tagName).to.equal("DIV");

    //    expect(divElem.outerHTML).to.equal('<div class="floating-label ng-scope" ng-class="{ \'has-error\' : f.xxx.$invalid &amp;&amp; f.xxx.$dirty }"><i class=\'fa fa-wifi\'></i>Server</div>');
    //});
});