﻿"use strict";

describe("SignalR Service", function () {    
    var signalRService, rootscope;
    var backendServerUrl = "http://localhost:8080";
    var backendLoggerUrl = "http://localhost:88";

    beforeEach(function() {
        module("portalApp.services", function($provide) {
            $provide.value("backendServerUrl", backendServerUrl);
            $provide.value("backendLoggerUrl", backendLoggerUrl);
        });

        inject(function (_signalRService_, $rootScope) {
            rootscope = $rootScope;
            signalRService = _signalRService_;
        });

        signalRService._private.feedConnection = {
            disconnected: function() {
            },

            start: function () {
                return { done: function() {} };
            },

            stop: function () {
                return { done: function () { } };
            }
        };
    });
 
    afterEach(function () {
        signalRService.stopConnections();
        mockSignalRClient.reset();
    });

    it("should be created successfully", function () {
        expect(signalRService).to.exist;
    });

    // #region ImageFeed Hub
    it("initializeFeedHub method should not create a new hub connection when the hub is already defined", function () {
        expect(signalRService).to.exist;

        signalRService._private.feedHub = "fakeValue";
        var request = new signalRService.FeedRequestOptions(0, false, false);
        signalRService.initializeFeedHub(request);

        expect(signalRService._private.feedHub).to.equal("fakeValue");
        expect(mockSignalRClient.createHubProxy).to.be.false;
        expect(mockSignalRClient.channelOn).to.be.false;
        expect(mockSignalRClient.channelInvoke).to.be.null;
        expect(mockSignalRClient.startDone).to.be.false;
        expect(mockSignalRClient.startFail).to.be.false;
        expect(mockSignalRClient.connectionStart).to.be.false;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
    });

    it("initializeFeedHub method should not create a new hub connection when the hub is already defined but invoke subscribe", function () {
        expect(signalRService).to.exist;

        var connection = $.hubConnection(backendServerUrl + "/signalr");

        signalRService._private.feedHub = connection.createHubProxy("ImageFeedHub");
        mockSignalRClient.createHubProxy = false; // reset flag before the call
        var request = new signalRService.FeedRequestOptions(1, true, true);
        signalRService.initializeFeedHub(request);

        expect(signalRService._private.feedHub).to.be.an('object');
        expect(mockSignalRClient.createHubProxy).to.be.false;
        expect(mockSignalRClient.channelOn).to.be.false;
        expect(mockSignalRClient.channelInvoke).to.equal("Subscribe");
        expect(mockSignalRClient.startDone).to.be.false;
        expect(mockSignalRClient.startFail).to.be.false;
        expect(mockSignalRClient.connectionStart).to.be.false;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
    });

    it("initializeFeedHub method should create a new hub connection but not invoke subscribe when the hub is not defined", function () {
        expect(signalRService).to.exist;

        signalRService._private.feedHub = null;
        var request = new signalRService.FeedRequestOptions(0, false, false);
        signalRService.initializeFeedHub(request);

        expect(signalRService._private.feedHub).to.be.an('object');
        expect(signalRService._private.feedHub).to.equal(mockSignalRClient.channel);
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.be.null;
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
    });

    it("initializeFeedHub method should create a new hub connection and invoke subscribe when the hub is not defined", function () {
        expect(signalRService).to.exist;

        signalRService._private.feedHub = null;
        var request = new signalRService.FeedRequestOptions(1, true, true);
        signalRService.initializeFeedHub(request);
        mockSignalRClient.doneFunc();

        expect(signalRService._private.feedHub).to.be.an('object');
        expect(signalRService._private.feedHub).to.equal(mockSignalRClient.channel);
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.equal("Subscribe");
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
    });

    it("startFeed method should not create a new hub connection and invoke subscribe when the hub is defined", function () {
        expect(signalRService).to.exist;

        signalRService._private.feedHub = null;
        var request = new signalRService.FeedRequestOptions(0, false, false);
        signalRService.initializeFeedHub(request);
        signalRService.startFeed();

        expect(signalRService._private.feedHub).to.be.an('object');
        expect(signalRService._private.feedHub).to.equal(mockSignalRClient.channel);
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.equal("Subscribe");
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
    });

    it("startFeed method should create a new hub connection and invoke subscribe when the hub is not defined", function () {
        expect(signalRService).to.exist;

        signalRService._private.feedHub = null;
        signalRService.startFeed();

        expect(signalRService._private.feedHub).to.be.an('object');
        expect(signalRService._private.feedHub).to.equal(mockSignalRClient.channel);
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.equal("Subscribe");
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
    });

    it("stopFeed method should not invoke subscribe when the hub is not defined", function () {
        expect(signalRService).to.exist;

        signalRService._private.feedHub = null;
        signalRService.stopFeed();

        expect(signalRService._private.feedHub).to.be.null;
        expect(mockSignalRClient.createHubProxy).to.be.false;
        expect(mockSignalRClient.channelOn).to.be.false;
        expect(mockSignalRClient.channelInvoke).to.be.null;
        expect(mockSignalRClient.startDone).to.be.false;
        expect(mockSignalRClient.startFail).to.be.false;
        expect(mockSignalRClient.connectionStart).to.be.false;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
    });

    it("stopFeed method should invoke subscribe when the hub is defined", function () {
        expect(signalRService).to.exist;

        signalRService._private.feedHub = null;
        var request = new signalRService.FeedRequestOptions(0, false, false);
        signalRService.initializeFeedHub(request);
        signalRService.stopFeed();

        expect(signalRService._private.feedHub).to.be.an('object');
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.equal("Subscribe");
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
    });

    it("NewImageFeedInfo event should be emitted on the rootScope", function () {
        expect(signalRService).to.exist;

        signalRService._private.feedHub = null;
        signalRService.startFeed();
        var messageReceived = null;
        var message = { message: "allo" };

        rootscope.$on("NewImageFeedInfo", function (e, m) {
            messageReceived = m;
        });

        mockSignalRClient.onFunctionDict["NewImageFeedInfo"](message); // call function defined to handle the event

        expect(signalRService._private.feedHub).to.be.an('object');
        expect(signalRService._private.feedHub).to.equal(mockSignalRClient.channel);
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.equal("Subscribe");
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
        expect(messageReceived).to.equal(message);
    });

    it("reduceFeed method should create a new hub connection and invoke subscribe when the hub is not defined", function () {
        expect(signalRService).to.exist;

        signalRService._private.feedHub = null;
        signalRService.reduceFeed();

        expect(signalRService._private.feedHub).to.be.an('object');
        expect(signalRService._private.feedHub).to.equal(mockSignalRClient.channel);
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.equal("Subscribe");
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
    });

    it("reduceFeed method should create a new hub connection and invoke subscribe when the hub is defined", function () {
        expect(signalRService).to.exist;

        signalRService._private.feedHub = null;
        var request = new signalRService.FeedRequestOptions(0, false, false);
        signalRService.initializeFeedHub(request);
        signalRService.reduceFeed();

        expect(signalRService._private.feedHub).to.be.an('object');
        expect(signalRService._private.feedHub).to.equal(mockSignalRClient.channel);
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.equal("Subscribe");
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
    });

    it("resetCounter method should invoke ResetCounter method from the image feed hub", function () {
        expect(signalRService).to.exist;

        signalRService._private.feedHub = null;
        var request = new signalRService.FeedRequestOptions(0, false, false);
        signalRService.initializeFeedHub(request);
        signalRService.resetCounter();

        expect(signalRService._private.feedHub).to.be.an('object');
        expect(signalRService._private.feedHub).to.equal(mockSignalRClient.channel);
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.equal("ResetCounter");
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
    });

    // #region Performance Hub
    it("initializePerformanceHub method should not create a new hub connection when the hub is already defined", function () {
        expect(signalRService).to.exist;

        signalRService._private.perfHub = "fakeValue";
        signalRService.initializePerformanceHub();

        expect(signalRService._private.perfHub).to.equal("fakeValue");
        expect(mockSignalRClient.createHubProxy).to.be.false;
        expect(mockSignalRClient.channelOn).to.be.false;
        expect(mockSignalRClient.channelInvoke).to.be.null;
        expect(mockSignalRClient.startDone).to.be.false;
        expect(mockSignalRClient.startFail).to.be.false;
        expect(mockSignalRClient.connectionStart).to.be.false;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
    });

    it("initializePerformanceHub method should create a new hub connection when the hub is undefined", function () {
        expect(signalRService).to.exist;

        signalRService._private.perfHub = null;
        signalRService.initializePerformanceHub();

        expect(signalRService._private.perfHub).to.be.an('object');
        expect(signalRService._private.perfHub).to.equal(mockSignalRClient.channel);
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.be.null;
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
    });

    it("CpuUsage event should be emitted on the rootScope", function () {
        expect(signalRService).to.exist;

        signalRService._private.perfHub = null;
        signalRService.initializePerformanceHub();
        var messageReceived = null;
        var message = { message: "allo" };

        rootscope.$on("CpuUsage", function (e, m) {
            messageReceived = m;
        });

        mockSignalRClient.onFunctionDict["CpuUsage"](message); // call function defined to handle the event

        expect(signalRService._private.perfHub).to.be.an('object');
        expect(signalRService._private.perfHub).to.equal(mockSignalRClient.channel);
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.be.null;
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
        expect(messageReceived).to.equal(message);
    });

    it("CpusUsage event should be emitted on the rootScope", function () {
        expect(signalRService).to.exist;

        signalRService._private.perfHub = null;
        signalRService.initializePerformanceHub();
        var messageReceived = null;
        var message = { message: "allo" };

        rootscope.$on("CpusUsage", function (e, m) {
            messageReceived = m;
        });

        mockSignalRClient.onFunctionDict["CpusUsage"](message); // call function defined to handle the event

        expect(signalRService._private.perfHub).to.be.an('object');
        expect(signalRService._private.perfHub).to.equal(mockSignalRClient.channel);
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.be.null;
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
        expect(messageReceived).to.equal(message);
    });

    it("DriveCSpace event should be emitted on the rootScope", function () {
        expect(signalRService).to.exist;

        signalRService._private.perfHub = null;
        signalRService.initializePerformanceHub();
        var messageReceived = null;
        var message = { message: "allo" };

        rootscope.$on("DriveCSpace", function (e, m) {
            messageReceived = m;
        });

        mockSignalRClient.onFunctionDict["DriveCSpace"](message); // call function defined to handle the event

        expect(signalRService._private.perfHub).to.be.an('object');
        expect(signalRService._private.perfHub).to.equal(mockSignalRClient.channel);
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.be.null;
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
        expect(messageReceived).to.equal(message);
    });

    it("DriveDSpace event should be emitted on the rootScope", function () {
        expect(signalRService).to.exist;

        signalRService._private.perfHub = null;
        signalRService.initializePerformanceHub();
        var messageReceived = null;
        var message = { message: "allo" };

        rootscope.$on("DriveDSpace", function (e, m) {
            messageReceived = m;
        });

        mockSignalRClient.onFunctionDict["DriveDSpace"](message); // call function defined to handle the event

        expect(signalRService._private.perfHub).to.be.an('object');
        expect(signalRService._private.perfHub).to.equal(mockSignalRClient.channel);
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.be.null;
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
        expect(messageReceived).to.equal(message);
    });

    it("DriveESpace event should be emitted on the rootScope", function () {
        expect(signalRService).to.exist;

        signalRService._private.perfHub = null;
        signalRService.initializePerformanceHub();
        var messageReceived = null;
        var message = { message: "allo" };

        rootscope.$on("DriveESpace", function (e, m) {
            messageReceived = m;
        });

        mockSignalRClient.onFunctionDict["DriveESpace"](message); // call function defined to handle the event

        expect(signalRService._private.perfHub).to.be.an('object');
        expect(signalRService._private.perfHub).to.equal(mockSignalRClient.channel);
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.be.null;
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
        expect(messageReceived).to.equal(message);
    });

    it("RamUsage event should be emitted on the rootScope", function () {
        expect(signalRService).to.exist;

        signalRService._private.perfHub = null;
        signalRService.initializePerformanceHub();
        var messageReceived = null;
        var message = { message: "allo" };

        rootscope.$on("RamUsage", function (e, m) {
            messageReceived = m;
        });

        mockSignalRClient.onFunctionDict["RamUsage"](message); // call function defined to handle the event

        expect(signalRService._private.perfHub).to.be.an('object');
        expect(signalRService._private.perfHub).to.equal(mockSignalRClient.channel);
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.be.null;
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
        expect(messageReceived).to.equal(message);
    });

    it("StorageInfo event should be emitted on the rootScope", function () {
        expect(signalRService).to.exist;

        signalRService._private.perfHub = null;
        signalRService.initializePerformanceHub();
        var messageReceived = null;
        var message = { message: "allo" };

        rootscope.$on("StorageInfo", function (e, m) {
            messageReceived = m;
        });

        mockSignalRClient.onFunctionDict["StorageInfo"](message); // call function defined to handle the event

        expect(signalRService._private.perfHub).to.be.an('object');
        expect(signalRService._private.perfHub).to.equal(mockSignalRClient.channel);
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.be.null;
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
        expect(messageReceived).to.equal(message);
    });

    // #region LastActivities Hub
    it("initializeLastActivitiesHub method should not create a new hub connection when the hub is already defined", function () {
        expect(signalRService).to.exist;

        signalRService._private.lastActivitiesHub = "fakeValue";
        signalRService.initializeLastActivitiesHub();

        expect(signalRService._private.lastActivitiesHub).to.equal("fakeValue");
        expect(mockSignalRClient.createHubProxy).to.be.false;
        expect(mockSignalRClient.channelOn).to.be.false;
        expect(mockSignalRClient.channelInvoke).to.be.null;
        expect(mockSignalRClient.startDone).to.be.false;
        expect(mockSignalRClient.startFail).to.be.false;
        expect(mockSignalRClient.connectionStart).to.be.false;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
    });

    it("LastActivities event should be emitted on the rootScope", function () {
        expect(signalRService).to.exist;

        signalRService._private.lastActivitiesHub = null;
        signalRService.initializeLastActivitiesHub();
        var messageReceived = null;
        var message = { message: "allo" };

        rootscope.$on("LastActivities", function (e, m) {
            messageReceived = m;
        });

        mockSignalRClient.onFunctionDict["LastActivities"](message); // call function defined to handle the event

        expect(signalRService._private.lastActivitiesHub).to.be.an('object');
        expect(signalRService._private.lastActivitiesHub).to.equal(mockSignalRClient.channel);
        expect(mockSignalRClient.createHubProxy).to.be.true;
        expect(mockSignalRClient.channelOn).to.be.true;
        expect(mockSignalRClient.channelInvoke).to.be.null;
        expect(mockSignalRClient.startDone).to.be.true;
        expect(mockSignalRClient.startFail).to.be.true;
        expect(mockSignalRClient.connectionStart).to.be.true;
        expect(mockSignalRClient.connectionStop).to.be.false;
        expect(mockSignalRClient.connectionError).to.be.false;
        expect(messageReceived).to.equal(message);
    });
});